@@ -17,12 +17,11 @@
 
 #include "config.h"
 #include <gio/gio.h>
-#define GETTEXT_PACKAGE "libparlatype"
+#define GETTEXT_PACKAGE GETTEXT_LIB
 #include <glib/gi18n-lib.h>
 #include <gst/gst.h>
 #include <gst/audio/streamvolume.h>
 #include "pt-i18n.h"
-#include "pt-waveloader.h"
 #include "pt-waveviewer.h"
 #include "pt-player.h"
 
@@ -30,6 +29,7 @@ struct _PtPlayerPrivate
 {
 	GstElement *play;
 	GstElement *scaletempo;
+	GstElement *volume_changer;
 	GstElement *play_bin;
 	GstElement *sphinx_bin;
 	GstElement *audio_bin;
@@ -41,6 +41,7 @@ struct _PtPlayerPrivate
 	gint64	    dur;
 	gdouble	    speed;
 	gdouble     volume;
+	gboolean    mute;
 	gint        pause;
 	gint        back;
 	gint        forward;
@@ -58,7 +59,6 @@ struct _PtPlayerPrivate
 	gchar          *timestamp_right;
 	gchar          *timestamp_sep;
 
-	PtWaveloader *wl;
 	PtWaveviewer *wv;
 };
 
@@ -66,6 +66,7 @@ enum
 {
 	PROP_0,
 	PROP_SPEED,
+	PROP_MUTE,
 	PROP_VOLUME,
 	PROP_TIMESTAMP_PRECISION,
 	PROP_TIMESTAMP_FIXED,
@@ -94,27 +95,14 @@ G_DEFINE_TYPE_WITH_PRIVATE (PtPlayer, pt_player, G_TYPE_OBJECT)
 /**
  * SECTION: pt-player
  * @short_description: The GStreamer backend for Parlatype.
- * @stability: Unstable
+ * @stability: Stable
  * @include: parlatype/pt-player.h
  *
  * PtPlayer is the GStreamer backend for Parlatype. Construct it with #pt_player_new().
- * Then you have to open a file, either with pt_player_open_uri_async() or
- * pt_player_open_uri(), the blocking version.
+ * Then you have to open a file with pt_player_open_uri().
  *
  * The internal time unit in PtPlayer are milliseconds and for scale widgets there
  * is a scale from 0 to 1000. Use it to jump to a position or to update your widget.
- *
- * While playing PtPlayer emits these signals:
- * - end-of-stream: End of file reached, in the GUI you might want to jump
- *    		       to the beginning, reset play button etc.
- * - error: A fatal error occured, the player is reset. There’s an error message.
- *
- * PtPlayer has two properties:
- * - speed: is a double from 0.5 to 1.5. 1.0 is normal playback, < 1.0 is slower,
- *     > 1.0 is faster. Changing the "speed" property doesn’t change playback though.
- *     Use the method instead.
- * - Volume is a double from 0 to 1. It can be set via the method or setting
-      the "volume" property.
  */
 
 
@@ -337,6 +325,34 @@ bus_call (GstBus     *bus,
 		g_signal_emit_by_name (player, "end-of-stream");
 		break;
 
+	case GST_MESSAGE_DURATION_CHANGED: {
+		gint64 dur;
+		gst_element_query_duration (player->priv->play, GST_FORMAT_TIME, &dur);
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "MESSAGE",
+				  "New duration: %" G_GINT64_FORMAT, dur);
+		if (player->priv->dur != player->priv->segend) {
+			player->priv->dur = dur;
+		} else {
+			player->priv->dur = player->priv->segend = dur;
+			pt_player_query_position (player, &pos);
+			pt_player_seek (player, pos);
+		}
+		break;
+		}
+	case GST_MESSAGE_STATE_CHANGED: {
+		if (GST_MESSAGE_SRC(msg) != GST_OBJECT (player->priv->play))
+			break;
+
+		gdouble volume;
+		volume = gst_stream_volume_get_volume (GST_STREAM_VOLUME (player->priv->volume_changer),
+			                               GST_STREAM_VOLUME_FORMAT_CUBIC);
+		if (player->priv->volume != volume) {
+			player->priv->volume = volume;
+			g_object_notify_by_pspec (G_OBJECT (player),
+			                          obj_properties[PROP_VOLUME]);
+		}
+		break;
+		}
 	case GST_MESSAGE_ERROR: {
 		gchar  *debug;
 		GError *error;
@@ -383,188 +399,10 @@ bus_call (GstBus     *bus,
 
 /* -------------------------- opening files --------------------------------- */
 
-
-static void
-propagate_progress_cb (PtWaveloader *wl,
-                       gdouble       progress,
-                       PtPlayer     *player)
-{
-	g_signal_emit_by_name (player, "load-progress", progress);
-}
-
-static void
-load_cb (PtWaveloader *wl,
-         GAsyncResult *res,
-         gpointer     *data)
-{
-	GTask    *task = (GTask *) data;
-	PtPlayer *player = g_task_get_source_object (task);
-	GError   *error = NULL;
-
-	if (pt_waveloader_load_finish (wl, res, &error)) {
-		player->priv->dur = player->priv->segend = pt_waveloader_get_duration (player->priv->wl);
-		player->priv->segstart = 0;
-
-		/* setup message handler */
-		add_message_bus (player);
-
-		pt_player_pause (player);
-
-		/* Block until state changed */
-		gst_element_get_state (
-			player->priv->play,
-			NULL, NULL,
-			GST_CLOCK_TIME_NONE);
-
-		metadata_get_position (player);
-		g_task_return_boolean (task, TRUE);
-
-	} else {
-		g_clear_object (&wl);
-		player->priv->wl = NULL;
-		g_task_return_error (task, error);
-	}
-
-	g_object_unref (player->priv->c);
-	g_object_unref (task);
-}
-
-/**
- * pt_player_open_uri_finish:
- * @player: a #PtPlayer
- * @result: the #GAsyncResult passed to your #GAsyncReadyCallback
- * @error: (nullable): a pointer to a NULL #GError, or NULL
- *
- * Gives the result of the async opening operation. A cancelled operation results
- * in an error, too.
- *
- * Return value: TRUE if successful, or FALSE with error set
- */
-gboolean
-pt_player_open_uri_finish (PtPlayer      *player,
-                           GAsyncResult  *result,
-                           GError       **error)
-{
-	g_return_val_if_fail (g_task_is_valid (result, player), FALSE);
-
-	return g_task_propagate_boolean (G_TASK (result), error);
-}
-
-/**
- * pt_player_open_uri_async:
- * @player: a #PtPlayer
- * @uri: the URI of the file
- * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
- * @user_data: (closure): user_data for callback
- *
- * Opens a local audio file for playback. It doesn’t work with videos or streams.
- * Only one file can be open at a time, playlists are not supported by the
- * backend. Opening a new file will close the previous one.
- *
- * When closing a file or on object destruction PtPlayer tries to write the
- * last position into the file’s metadata. On opening a file it reads the
- * metadata and jumps to the last known position if found.
- *
- * The player is set to the paused state and ready for playback. To start
- * playback use @pt_player_play().
- *
- * This is an asynchronous operation, to get the result call
- * pt_player_open_uri_finish() in your callback. For the blocking version see
- * pt_player_open_uri().
- *
- * While loading the file there is a #PtPlayer::load-progress signal emitted which stops before
- * reaching 100%. Don’t use it to determine whether the operation is finished.
- */
-void
-pt_player_open_uri_async (PtPlayer            *player,
-                          gchar               *uri,
-                          GAsyncReadyCallback  callback,
-                          gpointer             user_data)
-{
-	g_return_if_fail (PT_IS_PLAYER (player));
-	g_return_if_fail (uri != NULL);
-
-	GTask  *task;
-	GFile  *file;
-	gchar  *location = NULL;
-
-	task = g_task_new (player, NULL, callback, user_data);
-
-	/* Change uri to location */
-	file = g_file_new_for_uri (uri);
-	location = g_file_get_path (file);
-	
-	if (!location) {
-		g_task_return_new_error (
-				task,
-				GST_RESOURCE_ERROR,
-				GST_RESOURCE_ERROR_NOT_FOUND,
-				/* Translators: %s is a detailed error message. */
-				_("URI not valid: %s"), uri);
-		g_object_unref (file);
-		g_object_unref (task);
-		return;
-	}
-
-	if (!g_file_query_exists (file, NULL)) {
-		g_task_return_new_error (
-				task,
-				GST_RESOURCE_ERROR,
-				GST_RESOURCE_ERROR_NOT_FOUND,
-				/* Translators: %s is a detailed error message. */
-				_("File not found: %s"), location);
-		g_object_unref (file);
-		g_free (location);
-		g_object_unref (task);
-		return;
-	}
-
-	/* If we had an open file before, remember its position */
-	metadata_save_position (player);
-
-	/* Reset any open streams */
-	pt_player_clear (player);
-	player->priv->dur = -1;
-
-	g_object_set (G_OBJECT (player->priv->play), "uri", uri, NULL);
-	g_object_unref (file);
-	g_free (location);
-
-	g_clear_object (&player->priv->wl);
-	player->priv->wl = pt_waveloader_new (uri);
-	g_signal_connect (player->priv->wl,
-			 "progress",
-			 G_CALLBACK (propagate_progress_cb),
-			 player);
-
-	player->priv->c = g_cancellable_new ();
-	pt_waveloader_load_async (player->priv->wl,
-				  player->priv->c,
-				  (GAsyncReadyCallback) load_cb,
-				  task);
-}
-
-typedef struct
-{
-	GAsyncResult *res;
-	GMainLoop    *loop;
-} SyncData;
-
-static void
-quit_loop_cb (PtPlayer     *player,
-              GAsyncResult *res,
-              gpointer      user_data)
-{
-	SyncData *data = user_data;
-	data->res = g_object_ref (res);
-	g_main_loop_quit (data->loop);
-}
-
 /**
  * pt_player_open_uri:
  * @player: a #PtPlayer
  * @uri: the URI of the file
- * @error: (nullable): return location for an error, or NULL
  *
  * Opens a local audio file for playback. It doesn’t work with videos or streams.
  * Only one file can be open at a time, playlists are not supported by the
@@ -578,55 +416,48 @@ quit_loop_cb (PtPlayer     *player,
  * playback use @pt_player_play().
  *
  * This operation blocks until it is finished. It returns TRUE on success or
- * FALSE and an error. For the asynchronous version see
- * pt_player_open_uri_async().
+ * FALSE on error. Errors are emitted async via #PtPlayer::error signal.
  *
- * While loading the file there is a #PtPlayer::load-progress signal emitted.
- * However, it doesn’t emit 100%, the operation is finished when TRUE is returned.
+ * Return value: TRUE if successful, otherwise FALSE
  *
- * Return value: TRUE if successful, or FALSE with error set
+ * Since: 2.0
  */
 gboolean
 pt_player_open_uri (PtPlayer *player,
-                    gchar    *uri,
-                    GError  **error)
+                    gchar    *uri)
 {
 	g_return_val_if_fail (PT_IS_PLAYER (player), FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	gboolean      result;
-	SyncData      data;
-	GMainContext *context;
+	/* If we had an open file before, remember its position */
+	metadata_save_position (player);
 
-	context = g_main_context_new ();
-	g_main_context_push_thread_default (context);
+	/* Reset any open streams */
+	pt_player_clear (player);
+	player->priv->dur = -1;
 
-	data.loop = g_main_loop_new (context, FALSE);
-	data.res = NULL;
+	g_object_set (G_OBJECT (player->priv->play), "uri", uri, NULL);
 
-	pt_player_open_uri_async (player, uri, (GAsyncReadyCallback) quit_loop_cb, &data);
-	g_main_loop_run (data.loop);
+	/* setup message handler */
+	add_message_bus (player);
 
-	result = pt_player_open_uri_finish (player, data.res, error);
+	pt_player_pause (player);
 
-	g_main_context_unref (context);
-	g_main_loop_unref (data.loop);
-	g_object_unref (data.res);
+	/* Block until state changed, return on failure */
+	if (gst_element_get_state (player->priv->play,
+				   NULL, NULL,
+				   GST_CLOCK_TIME_NONE) == GST_STATE_CHANGE_FAILURE)
+		return FALSE;
 
-	return result;
-}
+	gint64 dur = 0;
+	gst_element_query_duration (player->priv->play, GST_FORMAT_TIME, &dur);
+	player->priv->dur = player->priv->segend = dur;
+	player->priv->segstart = 0;
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "MESSAGE",
+			  "Initial duration: %" G_GINT64_FORMAT, dur);
 
-/**
- * pt_player_cancel:
- * @player: a #PtPlayer
- *
- * Cancels the file opening operation, which triggers an error message.
- */
-void
-pt_player_cancel (PtPlayer *player)
-{
-	g_cancellable_cancel (player->priv->c);
+	metadata_get_position (player);
+	return TRUE;
 }
 
 
@@ -638,6 +469,8 @@ pt_player_cancel (PtPlayer *player)
  *
  * Sets the player to the paused state, meaning it stops playback and doesn’t
  * change position. To resume playback use @pt_player_play().
+ *
+ * Since: 1.4
  */
 void
 pt_player_pause (PtPlayer *player)
@@ -653,6 +486,8 @@ pt_player_pause (PtPlayer *player)
  *
  * Like @pt_player_pause(), additionally rewinds the value of
  * #PtPlayer:pause in milliseconds.
+ *
+ * Since: 1.6
  */
 void
 pt_player_pause_and_rewind (PtPlayer *player)
@@ -665,7 +500,11 @@ pt_player_pause_and_rewind (PtPlayer *player)
  * pt_player_get_pause:
  * @player: a #PtPlayer
  *
+ * Returns the value of #PtPlayer:pause.
+ *
  * Return value: time to rewind on pause in milliseconds
+ *
+ * Since: 1.6
  */
 gint
 pt_player_get_pause (PtPlayer *player)
@@ -682,6 +521,8 @@ pt_player_get_pause (PtPlayer *player)
  * Starts playback at the defined speed until it reaches the end of stream (or
  * the end of the selection). If the current position is at the end, playback
  * will start from the beginning of stream or selection.
+ *
+ * Since: 1.4
  */
 void
 pt_player_play (PtPlayer *player)
@@ -727,6 +568,8 @@ pt_player_play (PtPlayer *player)
  * @player: a #PtPlayer
  *
  * Toggles between playback and pause, rewinds on pause.
+ *
+ * Since: 1.6
  */
 void
 pt_player_play_pause (PtPlayer *player)
@@ -742,17 +585,19 @@ pt_player_play_pause (PtPlayer *player)
 
 	switch (state) {
 	case GST_STATE_NULL:
-		/* fall through */
+		/* If we don't do anything, make sure to return	without emitting
+		 * a signal. It would toggle the play button in the GUI. */
+		return;
 	case GST_STATE_PAUSED:
 		pt_player_play (player);
 		break;
 	case GST_STATE_PLAYING:
 		pt_player_pause_and_rewind (player);
+		break;
 	case GST_STATE_VOID_PENDING:
-		/* fall through */
+		return;
 	case GST_STATE_READY:
-		/* don’t know what to do */
-		;
+		return;
 	}
 
 	g_signal_emit_by_name (player, "play-toggled");
@@ -769,6 +614,8 @@ pt_player_play_pause (PtPlayer *player)
  * be set to the selection’s start position, otherwise the current position is
  * not changed. Playing will end at the stop position and it’s not possible to
  * jump out of the selection until it is cleared with #pt_player_clear_selection.
+ *
+ * Since: 1.5
  */
 void
 pt_player_set_selection (PtPlayer *player,
@@ -797,6 +644,8 @@ pt_player_set_selection (PtPlayer *player,
  * @player: a #PtPlayer
  *
  * Clear and reset any selection.
+ *
+ * Since: 1.5
  */
 void
 pt_player_clear_selection (PtPlayer *player)
@@ -818,7 +667,11 @@ pt_player_clear_selection (PtPlayer *player)
  * pt_player_selection_active:
  * @player: a #PtPlayer
  *
+ * Returns whether there is currently a selection set or not.
+ *
  * Return value: TRUE if there is a selection
+ *
+ * Since: 1.6
  */
 gboolean
 pt_player_selection_active (PtPlayer *player)
@@ -838,6 +691,8 @@ pt_player_selection_active (PtPlayer *player)
  *
  * <note><para>Note that depending on the file/stream format this works more
  * or less good.</para></note>
+ *
+ * Since: 1.5
  */
 void
 pt_player_rewind (PtPlayer *player,
@@ -879,6 +734,8 @@ pt_player_rewind (PtPlayer *player,
  * @speed: the speed
  *
  * Play fast forward at the given speed.
+ *
+ * Since: 1.5
  */
 void
 pt_player_fast_forward (PtPlayer *player,
@@ -923,6 +780,8 @@ pt_player_fast_forward (PtPlayer *player,
  * to the end of stream (or selection). A negative value means jumping back.
  * If the resulting position would be negative (or before the selection), it
  * jumps to position 0:00 (or to the start of the selection).
+ *
+ * Since: 1.4
  */
 void
 pt_player_jump_relative (PtPlayer *player,
@@ -957,6 +816,8 @@ pt_player_jump_relative (PtPlayer *player,
  * @player: a #PtPlayer
  *
  * Jumps back the value of #PtPlayer:back.
+ *
+ * Since: 1.6
  */
 void
 pt_player_jump_back (PtPlayer *player)
@@ -975,6 +836,8 @@ pt_player_jump_back (PtPlayer *player)
  * @player: a #PtPlayer
  *
  * Jumps forward the value of #PtPlayer:forward.
+ *
+ * Since: 1.6
  */
 void
 pt_player_jump_forward (PtPlayer *player)
@@ -987,7 +850,11 @@ pt_player_jump_forward (PtPlayer *player)
  * pt_player_get_back:
  * @player: a #PtPlayer
  *
+ * Returns the value of #PtPlayer:back.
+ *
  * Return value: time to jump back in milliseconds
+ *
+ * Since: 1.6
  */
 gint
 pt_player_get_back (PtPlayer *player)
@@ -1001,7 +868,11 @@ pt_player_get_back (PtPlayer *player)
  * pt_player_get_forward:
  * @player: a #PtPlayer
  *
+ * Returns the value of #PtPlayer:forward.
+ *
  * Return value: time to jump forward in milliseconds
+ *
+ * Since: 1.6
  */
 gint
 pt_player_get_forward (PtPlayer *player)
@@ -1019,6 +890,8 @@ pt_player_get_forward (PtPlayer *player)
  * Jumps to a given position in stream. The position is given in @milliseconds
  * starting from position 0:00. A position beyond the duration of stream (or
  * outside the selection) is ignored.
+ *
+ * Since: 1.4
  */
 void
 pt_player_jump_to_position (PtPlayer *player,
@@ -1030,6 +903,15 @@ pt_player_jump_to_position (PtPlayer *player,
 
 	pos = GST_MSECOND * (gint64) milliseconds;
 
+	if (pos < 0) {
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+				  "MESSAGE", "Jump to position failed: negative value");
+		return;
+	}
+
+	/* TODO on opening a new file and jumping to the initial position,
+	 * sometimes there is no duration yet and the jump is not done. */
+
 	if (pos > player->priv->segend || pos < player->priv->segstart) {
 		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
 				  "MESSAGE", "Jump to position failed: start = %" G_GINT64_FORMAT, player->priv->segstart);
@@ -1052,6 +934,8 @@ pt_player_jump_to_position (PtPlayer *player,
  * at 1000. This will jump to the given position. If your widget uses a different
  * scale, it’s up to you to convert it to 1/1000. Values beyond 1000 are not
  * allowed, values outside the selection are ignored.
+ *
+ * Since: 1.4
  */
 void
 pt_player_jump_to_permille (PtPlayer *player,
@@ -1080,6 +964,8 @@ pt_player_jump_to_permille (PtPlayer *player,
  * Failure in querying the position returns -1.
  *
  * Return value: a scale position between 0 and 1000 or -1 on failure
+ *
+ * Since: 1.4
  */
 gint
 pt_player_get_permille (PtPlayer *player)
@@ -1105,9 +991,7 @@ pt_player_get_permille (PtPlayer *player)
  * Recommended speed is starting from 0.5 as quality is rather poor below that.
  * Parlatype doesn’t change the pitch during slower or faster playback.
  *
- * Note: If you want to change the speed during playback, you have to use this
- * method. Changing the "speed" property of PtPlayer, will take effect only
- * later.
+ * Since: 1.4
  */
 void
 pt_player_set_speed (PtPlayer *player,
@@ -1118,13 +1002,47 @@ pt_player_set_speed (PtPlayer *player,
 
 	gint64 pos;
 
-	if (!pt_player_query_position (player, &pos))
+	player->priv->speed = speed;
+
+	/* on object construction there is no pipeline yet */
+	if (player->priv->play == NULL)
 		return;
 
-	player->priv->speed = speed;
+	if (pt_player_query_position (player, &pos))
+		pt_player_seek (player, pos);
+
 	g_object_notify_by_pspec (G_OBJECT (player),
 				  obj_properties[PROP_SPEED]);
-	pt_player_seek (player, pos);
+}
+
+/**
+ * pt_player_get_volume:
+ * @player: a #PtPlayer
+ *
+ * Gets the volume on a scale between 0 and 1.
+ *
+ * Return value: the current volume
+ *
+ * Since: 2.1
+ */
+gdouble
+pt_player_get_volume (PtPlayer *player)
+{
+	g_return_val_if_fail (PT_IS_PLAYER (player), -1);
+
+	gdouble volume;
+
+	/* Pulseaudio sink does not propagate volume changes in GST_STATE_PAUSED
+	 * or lower. Ask for real value and update cached volume. */
+
+	if (player->priv->play) {
+		volume = gst_stream_volume_get_volume (
+				GST_STREAM_VOLUME (player->priv->volume_changer),
+				GST_STREAM_VOLUME_FORMAT_CUBIC);
+		if (player->priv->volume != volume)
+			player->priv->volume = volume;
+	}
+	return player->priv->volume;
 }
 
 /**
@@ -1134,6 +1052,8 @@ pt_player_set_speed (PtPlayer *player,
  *
  * Sets the volume on a scale between 0 and 1. Instead of using this method
  * you could set the "volume" property.
+ *
+ * Since: 1.4
  */
 void
 pt_player_set_volume (PtPlayer *player,
@@ -1142,14 +1062,60 @@ pt_player_set_volume (PtPlayer *player,
 	g_return_if_fail (PT_IS_PLAYER (player));
 	g_return_if_fail (volume >= 0 && volume <= 1);
 
-	gst_stream_volume_set_volume (GST_STREAM_VOLUME (player->priv->play),
-	                              GST_STREAM_VOLUME_FORMAT_CUBIC,
-	                              volume);
 	player->priv->volume = volume;
+
+	if (player->priv->play)
+		gst_stream_volume_set_volume (GST_STREAM_VOLUME (player->priv->volume_changer),
+			                      GST_STREAM_VOLUME_FORMAT_CUBIC,
+			                      volume);
+
 	g_object_notify_by_pspec (G_OBJECT (player),
 				  obj_properties[PROP_VOLUME]);
 }
 
+/**
+ * pt_player_get_mute:
+ * @player: a #PtPlayer
+ *
+ * Get mute state of the audio stream.
+
+ * Return value: TRUE for muted state, FALSE for normal state
+ *
+ * Since: 2.1
+ */
+gboolean
+pt_player_get_mute (PtPlayer *player)
+{
+	g_return_val_if_fail (PT_IS_PLAYER (player), FALSE);
+
+	gboolean retval = FALSE;
+
+	if (player->priv->play)
+		retval = gst_stream_volume_get_mute (GST_STREAM_VOLUME (player->priv->volume_changer));
+
+	return retval;
+}
+
+/**
+ * pt_player_set_mute:
+ * @player: a #PtPlayer
+ * @mute: a gboolean
+ *
+ * Mute the player (with TRUE) or set it back to normal volume (with FALSE).
+ * This remembers the volume level, so you don’t have to keep track of the old value.
+ *
+ * Since: 2.1
+ */
+void
+pt_player_set_mute (PtPlayer *player,
+                    gboolean  mute)
+{
+	g_return_if_fail (PT_IS_PLAYER (player));
+
+	if (player->priv->play)
+		gst_stream_volume_set_mute (GST_STREAM_VOLUME (player->priv->volume_changer), mute);
+}
+
 /**
  * pt_player_mute_volume:
  * @player: a #PtPlayer
@@ -1157,14 +1123,15 @@ pt_player_set_volume (PtPlayer *player,
  *
  * Mute the player (with TRUE) or set it back to normal volume (with FALSE).
  * This remembers the volume level, so you don’t have to keep track of the old value.
+ *
+ * Since: 1.4
+ * Deprecated: 2.1: Use pt_player_set_mute() instead.
  */
 void
 pt_player_mute_volume (PtPlayer *player,
                        gboolean  mute)
 {
-	g_return_if_fail (PT_IS_PLAYER (player));
-
-	gst_stream_volume_set_mute (GST_STREAM_VOLUME (player->priv->play), mute);
+	pt_player_set_mute (player, mute);
 }
 
 /**
@@ -1174,6 +1141,8 @@ pt_player_mute_volume (PtPlayer *player,
  * Returns the current position in stream.
  *
  * Return value: position in milliseconds or -1 on failure
+ *
+ * Since: 1.5
  */
 gint64
 pt_player_get_position (PtPlayer *player)
@@ -1195,6 +1164,8 @@ pt_player_get_position (PtPlayer *player)
  * Returns the duration of stream.
  *
  * Return value: duration in milliseconds
+ *
+ * Since: 1.5
  */
 gint64
 pt_player_get_duration (PtPlayer *player)
@@ -1205,29 +1176,6 @@ pt_player_get_duration (PtPlayer *player)
 }
 
 
-/* ------------------------- Waveform stuff --------------------------------- */
-
-/**
- * pt_player_get_data:
- * @player: a #PtPlayer
- * @pps: the requested pixel per second ratio
- *
- * Returns wave data needed for visual representation as raw data. The
- * requested resolution is given as pixel per seconds, e.g. 100 means one second
- * is represented by 100 samples, is 100 pixels wide. The returned resolution
- * doesn’t have to be necessarily exactly the requested resolution, it might be
- * a bit differnt, depending on the bit rate.
- *
- * Return value: (transfer full): the #PtWavedata
- */
-PtWavedata*
-pt_player_get_data (PtPlayer *player,
-                    gint      pps)
-{
-	return pt_waveloader_get_data (player->priv->wl, pps);
-}
-
-
 /* --------------------- Other widgets -------------------------------------- */
 
 static void
@@ -1292,6 +1240,8 @@ wv_play_toggled_cb (GtkWidget *widget,
  *
  * Connect a #PtWaveviewer. The #PtPlayer will monitor selections made in the
  * #PtWaveviewer and act accordingly.
+ *
+ * Since: 1.6
  */
 void
 pt_player_connect_waveviewer (PtPlayer     *player,
@@ -1326,6 +1276,8 @@ pt_player_connect_waveviewer (PtPlayer     *player,
  * Returns the URI of the currently open file or NULL if it can’t be determined.
  *
  * Return value: (transfer full): the uri
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_uri (PtPlayer *player)
@@ -1344,6 +1296,8 @@ pt_player_get_uri (PtPlayer *player)
  * Returns the display name of the currently open file or NULL if it can’t be determined.
  *
  * Return value: (transfer full): the file name
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_filename (PtPlayer *player)
@@ -1406,6 +1360,8 @@ pt_player_get_filename (PtPlayer *player)
  * in long format, too.
  *
  * Return value: (transfer full): the time string
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_time_string (gint            time,
@@ -1527,6 +1483,8 @@ pt_player_get_time_string (gint            time,
  * If the current position can not be determined, NULL is returned.
  *
  * Return value: (transfer full): the time string
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_current_time_string (PtPlayer        *player,
@@ -1556,6 +1514,8 @@ pt_player_get_current_time_string (PtPlayer        *player,
  * If the duration can not be determined, NULL is returned.
  *
  * Return value: (transfer full): the time string
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_duration_time_string (PtPlayer        *player,
@@ -1586,6 +1546,8 @@ pt_player_get_duration_time_string (PtPlayer        *player,
  * #PtPlayer:timestamp-fraction-sep and #PtPlayer:timestamp-delimiter.
  *
  * Return value: (transfer full): the timestamp
+ *
+ * Since: 1.6
  */
 gchar*
 pt_player_get_timestamp_for_time (PtPlayer *player,
@@ -1667,6 +1629,8 @@ pt_player_get_timestamp_for_time (PtPlayer *player,
  * If the current position can not be determined, NULL is returned.
  *
  * Return value: (transfer full): the timestamp
+ *
+ * Since: 1.4
  */
 gchar*
 pt_player_get_timestamp (PtPlayer *player)
@@ -1695,6 +1659,8 @@ pt_player_get_timestamp (PtPlayer *player)
  *
  * Return value: the time in milliseconds represented by the timestamp or -1
  * for invalid timestamps
+ *
+ * Since: 1.6
  */
 gint
 pt_player_get_timestamp_position (PtPlayer *player,
@@ -1813,6 +1779,8 @@ pt_player_get_timestamp_position (PtPlayer *player,
  * position immediately after.
  *
  * Return value: TRUE if the timestamp is valid, FALSE if not
+ *
+ * Since: 1.4
  */
 gboolean
 pt_player_string_is_timestamp (PtPlayer *player,
@@ -1834,6 +1802,8 @@ pt_player_string_is_timestamp (PtPlayer *player,
  * valid timestamp.
  *
  * Return value: TRUE on success, FALSE if the timestamp is not valid
+ *
+ * Since: 1.4
  */
 gboolean
 pt_player_goto_timestamp (PtPlayer *player,
@@ -1866,6 +1836,7 @@ pt_player_init (PtPlayer *player)
 	player->priv->wv = NULL;
 	player->sphinx = NULL;
 	player->priv->scaletempo = NULL;
+	player->priv->volume_changer = NULL;
 	player->priv->play_bin = NULL;
 	player->priv->sphinx_bin = NULL;
 }
@@ -1875,7 +1846,7 @@ notify_volume_idle_cb (PtPlayer *player)
 {
 	gdouble vol;
 
-	vol = gst_stream_volume_get_volume (GST_STREAM_VOLUME (player->priv->play),
+	vol = gst_stream_volume_get_volume (GST_STREAM_VOLUME (player->priv->volume_changer),
 	                                    GST_STREAM_VOLUME_FORMAT_CUBIC);
 	player->priv->volume = vol;
 	g_object_notify_by_pspec (G_OBJECT (player),
@@ -1896,6 +1867,28 @@ vol_changed (GObject    *object,
 	g_source_set_name_by_id (id, "[parlatype] notify_volume_idle_cb");
 }
 
+static gboolean
+notify_mute_idle_cb (PtPlayer *player)
+{
+	gboolean mute;
+
+	mute = gst_stream_volume_get_mute (GST_STREAM_VOLUME (player->priv->volume_changer));
+	player->priv->mute = mute;
+	g_object_notify_by_pspec (G_OBJECT (player),
+	                          obj_properties[PROP_MUTE]);
+	return FALSE;
+}
+
+static void
+mute_changed (GObject    *object,
+              GParamSpec *pspec,
+              PtPlayer   *player)
+{
+	guint id;
+	id = g_idle_add ((GSourceFunc) notify_mute_idle_cb, player);
+	g_source_set_name_by_id (id, "[parlatype] notify_mute_idle_cb");
+}
+
 static GstElement*
 make_element (gchar   *factoryname,
               gchar   *name,
@@ -1963,6 +1956,27 @@ create_sphinx_bin (PtPlayer  *player,
 }
 #endif
 
+#ifndef G_OS_WIN32
+static gboolean
+have_pulseaudio_server (void)
+{
+	/* Adapted from Quod Libet ...quodlibet/player/gstbe/util.py:
+	 * If we have a pulsesink we can get the server presence through
+	 * setting the ready state */
+	GstElement           *pulse;
+	GstStateChangeReturn  state;
+
+	pulse = gst_element_factory_make ("pulsesink", NULL);
+	if (!pulse)
+		return FALSE;
+	gst_element_set_state (pulse, GST_STATE_READY);
+	state = gst_element_get_state (pulse, NULL, NULL, GST_CLOCK_TIME_NONE);
+	gst_element_set_state (pulse, GST_STATE_NULL);
+	gst_object_unref (pulse);
+	return (state != GST_STATE_CHANGE_FAILURE);
+}
+#endif
+
 static GstElement*
 create_play_bin (PtPlayer  *player,
                  GError   **error)
@@ -1970,21 +1984,68 @@ create_play_bin (PtPlayer  *player,
 	GError *earlier_error = NULL;
 
 	/* Create gstreamer elements */
-	GstElement *capsfilter;
-	GstElement *audiosink;
-	GstElement *queue;
+	GstElement   *capsfilter;
+	GstElement   *audiosink;
+	GstElement   *queue;
+	gchar        *sink;
+	GObjectClass *sinkclass;
+	GParamSpec   *has_volume;
+	GParamSpec   *has_mute;
 
 	capsfilter = make_element ("capsfilter", "audiofilter", &earlier_error);
 	/* defined error propagation; skipping any cleanup */
 	PROPAGATE_ERROR_NULL
-	audiosink = make_element ("autoaudiosink", "audiosink", &earlier_error);
-	PROPAGATE_ERROR_NULL
+
 	queue = make_element ("queue", "player_queue", &earlier_error);
 	PROPAGATE_ERROR_NULL
 
+	/* Choose an audiosink ourselves instead of relying on autoaudiosink.
+	 * It chose waveformsink on win32 (not a good choice) and it will be
+	 * a fakesink until the first stream is loaded, so we can't query the
+	 * sinks properties until then. */
+
+#ifdef G_OS_WIN32
+	sink = "directsoundsink";
+#else
+	if (have_pulseaudio_server ())
+		sink = "pulsesink";
+	else
+		sink = "alsasink";
+#endif
+	audiosink = make_element (sink, "audiosink", &earlier_error);
+	PROPAGATE_ERROR_NULL
+
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
+		          "MESSAGE", "Audio sink is %s", sink);
+
+	/* Audiosinks without a "volume" property can be controlled by the
+	 * playbin element, but there is a noticable delay in setting the volume.
+	 * Create a "volume" element for those audiosinks.
+	 * Note: Query is generic, it's actually only for alsasink.
+	 * The exception for directsoundsink is because "mute" doesn't work. */
+
+	sinkclass = G_OBJECT_GET_CLASS(audiosink);
+	has_volume = g_object_class_find_property (sinkclass, "volume");
+	has_mute =   g_object_class_find_property (sinkclass, "mute");
+
+	if ((!has_volume && !has_mute) || g_strcmp0 (sink, "directsoundsink") == 0) {
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
+		                  "MESSAGE", "Creating a \"volume\" element for %s", sink);
+		player->priv->volume_changer = make_element ("volume", "volume", &earlier_error);
+		PROPAGATE_ERROR_NULL
+	}
+
 	GstElement *audio = gst_bin_new ("player-audiobin");
-	gst_bin_add_many (GST_BIN (audio), queue, capsfilter, audiosink, NULL);
-	gst_element_link_many (queue, capsfilter, audiosink, NULL);
+	if (player->priv->volume_changer) {
+		gst_bin_add_many (GST_BIN (audio), queue, capsfilter,
+				  player->priv->volume_changer, audiosink, NULL);
+		gst_element_link_many (queue, capsfilter,
+				       player->priv->volume_changer, audiosink, NULL);
+	} else {
+		gst_bin_add_many (GST_BIN (audio), queue, capsfilter, audiosink, NULL);
+		gst_element_link_many (queue, capsfilter, audiosink, NULL);
+		player->priv->volume_changer = audiosink;
+	}
 
 	/* create ghost pad for audiosink */
 	GstPad *audiopad = gst_element_get_static_pad (queue, "sink");
@@ -2027,6 +2088,8 @@ Note 3: The original intent was to dynamically switch the tee element to either
         omitted. On the other hand, maybe they could be somehow synced to have
         audio and recognition in real time.
 
+Note 4: autoaudiosink (in the upper right corner) was replaced by a chosen sink
+        and an optional "volume" element.
 */
 
 static void
@@ -2079,6 +2142,8 @@ pt_player_setup_pipeline (PtPlayer  *player,
 	   Syncing is done only in Play state */
 	g_signal_connect (G_OBJECT (player->priv->play),
 			"notify::volume", G_CALLBACK (vol_changed), player);
+	g_signal_connect (G_OBJECT (player->priv->play),
+			"notify::mute", G_CALLBACK (mute_changed), player);
 	return TRUE;
 }
 
@@ -2130,6 +2195,8 @@ add_element (GstBin     *parent,
  * the results. Start recognition with pt_player_play().
  *
  * Return value: TRUE on success, FALSE if the pipeline could not be set up
+ *
+ * Since: 1.6
  */
 gboolean
 pt_player_setup_sphinx (PtPlayer  *player,
@@ -2170,6 +2237,8 @@ pt_player_setup_sphinx (PtPlayer  *player,
  * do anything with the #PtPlayer before calling the setup function.
  *
  * Return value: TRUE on success, FALSE if the pipeline could not be set up
+ *
+ * Since: 1.6
  */
 gboolean
 pt_player_setup_player (PtPlayer  *player,
@@ -2198,7 +2267,8 @@ pt_player_setup_player (PtPlayer  *player,
 			"audio-filter", player->priv->scaletempo, NULL);
 
 	pt_player_set_state_blocking (player, GST_STATE_PAUSED);
-	pt_player_jump_to_position (player, pos);
+	if (pos > 0)
+		pt_player_jump_to_position (player, pos);
 
 	return TRUE;
 }
@@ -2206,8 +2276,7 @@ pt_player_setup_player (PtPlayer  *player,
 static void
 pt_player_dispose (GObject *object)
 {
-	PtPlayer *player;
-	player = PT_PLAYER (object);
+	PtPlayer *player = PT_PLAYER (object);
 
 	if (player->priv->play) {
 		/* remember position */
@@ -2231,8 +2300,6 @@ pt_player_dispose (GObject *object)
 		gst_object_unref (GST_OBJECT (player->priv->tee_sphinxpad));
 	}
 
-	g_clear_object (&player->priv->wl);
-
 	G_OBJECT_CLASS (pt_player_parent_class)->dispose (object);
 }
 
@@ -2242,22 +2309,18 @@ pt_player_set_property (GObject      *object,
                         const GValue *value,
                         GParamSpec   *pspec)
 {
-	PtPlayer *player;
-	player = PT_PLAYER (object);
-	gdouble tmp;
+	PtPlayer *player = PT_PLAYER (object);
 	const gchar *tmpchar;
 
 	switch (property_id) {
 	case PROP_SPEED:
-		player->priv->speed = g_value_get_double (value);
+		pt_player_set_speed (player, g_value_get_double (value));
 		break;
 	case PROP_VOLUME:
-		tmp = g_value_get_double (value);
-		if (player->priv->play)
-			gst_stream_volume_set_volume (GST_STREAM_VOLUME (player->priv->play),
-				                      GST_STREAM_VOLUME_FORMAT_CUBIC,
-				                      tmp);
-		player->priv->volume = tmp;
+		pt_player_set_volume (player, g_value_get_double (value));
+		break;
+	case PROP_MUTE:
+		pt_player_set_mute (player, g_value_get_boolean (value));
 		break;
 	case PROP_TIMESTAMP_PRECISION:
 		player->priv->timestamp_precision = g_value_get_int (value);
@@ -2318,15 +2381,17 @@ pt_player_get_property (GObject    *object,
                         GValue     *value,
                         GParamSpec *pspec)
 {
-	PtPlayer *player;
-	player = PT_PLAYER (object);
+	PtPlayer *player = PT_PLAYER (object);
 
 	switch (property_id) {
 	case PROP_SPEED:
 		g_value_set_double (value, player->priv->speed);
 		break;
 	case PROP_VOLUME:
-		g_value_set_double (value, player->priv->volume);
+		g_value_set_double (value, pt_player_get_volume (player));
+		break;
+	case PROP_MUTE:
+		g_value_set_boolean (value, player->priv->mute);
 		break;
 	case PROP_TIMESTAMP_PRECISION:
 		g_value_set_int (value, player->priv->timestamp_precision);
@@ -2368,25 +2433,6 @@ pt_player_class_init (PtPlayerClass *klass)
 	G_OBJECT_CLASS (klass)->get_property = pt_player_get_property;
 	G_OBJECT_CLASS (klass)->dispose = pt_player_dispose;
 
-	/**
-	* PtPlayer::load-progress:
-	* @player: the player emitting the signal
-	* @progress: the new progress state, ranging from 0.0 to 1.0
-	*
-	* Indicates progress on a scale from 0.0 to 1.0, however it does not
-	* emit the value 0.0 nor 1.0. Wait for a TRUE player-state-changed
-	* signal or an error signal to dismiss a gui element showing progress.
-	*/
-	g_signal_new ("load-progress",
-		      PT_TYPE_PLAYER,
-		      G_SIGNAL_RUN_FIRST,
-		      0,
-		      NULL,
-		      NULL,
-		      g_cclosure_marshal_VOID__DOUBLE,
-		      G_TYPE_NONE,
-		      1, G_TYPE_DOUBLE);
-
 	/**
 	* PtPlayer::end-of-stream:
 	* @player: the player emitting the signal
@@ -2531,10 +2577,16 @@ pt_player_class_init (PtPlayerClass *klass)
 			G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
 	/**
-	* PtPlayer:volume:
-	*
-	* The volume for playback.
-	*/
+	 * PtPlayer:volume:
+	 *
+	 * The volume for playback.
+	 *
+	 * <note><para>Pulseaudio sink does not propagate volume changes at GST_STATE_PAUSED
+	 * or lower. This property will notify of changes only in GST_STATE_PLAYING and
+	 * on state changes, e.g. from GST_STATE_READY to GST_STATE_PAUSED. Getting this
+	 * property returns the real current volume level even if there was no notification before.
+	 * </para></note>
+	 */
 	obj_properties[PROP_VOLUME] =
 	g_param_spec_double (
 			"volume",
@@ -2545,6 +2597,19 @@ pt_player_class_init (PtPlayerClass *klass)
 			1.0,
 			G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
+	/**
+	* PtPlayer:mute:
+	*
+	* Mute state of the audio stream.
+	*/
+	obj_properties[PROP_MUTE] =
+	g_param_spec_boolean (
+			"mute",
+			"Mute",
+			"Mute state of audio stream",
+			FALSE,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
 	/**
 	* PtPlayer:timestamp-precision:
 	*
@@ -2690,6 +2755,8 @@ pt_player_class_init (PtPlayerClass *klass)
  * After use g_object_unref() it.
  *
  * Return value: (transfer full): a new pt_player
+ *
+ * Since: 1.6
  */
 PtPlayer *
 pt_player_new (void)
