@@ -26,9 +26,10 @@
 #include "config.h"
 #include <math.h>	/* fabs */
 #include <string.h>
-#define GETTEXT_PACKAGE "libparlatype"
+#define GETTEXT_PACKAGE GETTEXT_LIB
 #include <glib/gi18n-lib.h>
 #include "pt-marshalers.h"
+#include "pt-waveloader.h"
 #include "pt-waveviewer-ruler.h"
 #include "pt-waveviewer-waveform.h"
 #include "pt-waveviewer-selection.h"
@@ -43,7 +44,8 @@
 
 struct _PtWaveviewerPrivate {
 	/* Wavedata */
-	gfloat   *peaks;
+	PtWaveloader *loader;
+	GArray   *peaks;
 	gint64    peaks_size;	/* size of array */
 	gint      px_per_sec;
 	gint64    duration;	/* in milliseconds */
@@ -55,6 +57,7 @@ struct _PtWaveviewerPrivate {
 	gboolean  show_ruler;
 	gboolean  has_selection;
 	gboolean  zoom;
+	gint      pps;
 
 	gint64    zoom_time;
 	gint      zoom_pos;
@@ -77,6 +80,14 @@ struct _PtWaveviewerPrivate {
 	GtkWidget  *focus;
 	GtkWidget  *cursor;
 	GtkWidget  *selection;
+
+	/* Event handling */
+	GtkGesture *button;
+#if GTK_CHECK_VERSION(3,24,0)
+	GtkEventController *motion_ctrl;
+#endif
+
+	guint       tick_handler;
 };
 
 enum
@@ -89,7 +100,7 @@ enum
 	PROP_HAS_SELECTION,
 	PROP_SELECTION_START,
 	PROP_SELECTION_END,
-	PROP_ZOOM,
+	PROP_PPS,
 	N_PROPERTIES
 };
 
@@ -113,10 +124,11 @@ G_DEFINE_TYPE_WITH_PRIVATE (PtWaveviewer, pt_waveviewer, GTK_TYPE_SCROLLED_WINDO
 /**
  * SECTION: pt-waveviewer
  * @short_description: A GtkWidget to display a waveform.
- * @stability: Unstable
+ * @stability: Stable
  * @include: parlatype/pt-waveviewer.h
  *
- * Displays a waveform provided by #PtPlayer or #PtWaveloader.
+ * Displays a waveform and lets the user interact with it: jumping to a position,
+ * make selections and so on.
  */
 
 
@@ -311,7 +323,7 @@ pt_waveviewer_key_press_event (GtkWidget   *widget,
 	if (gdk_event_get_event_type ((GdkEvent*) event) != GDK_KEY_PRESS)
 		return FALSE;
 
-	if (!self->priv->peaks)
+	if (self->priv->peaks->len == 0)
 		return FALSE;
 
 	gdk_event_get_state ((GdkEvent*) event, &state);
@@ -439,30 +451,30 @@ set_cursor (GtkWidget *widget,
 }
 
 static gboolean
-pt_waveviewer_button_press_event (GtkWidget      *widget,
-                                  GdkEventButton *event)
+pt_waveviewer_button_press_event (GtkGestureMultiPress *gesture,
+                                  gint                  n_press,
+                                  gdouble               x,
+                                  gdouble               y,
+                                  gpointer              user_data)
 {
-	PtWaveviewer *self = PT_WAVEVIEWER (widget);
-	GdkEventType         type;
+	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
 	GdkModifierType      state;
 	guint                button;
-	gdouble              x;
 	gint64               clicked;	/* the sample clicked on */
 	gint64               pos;	/* clicked sample’s position in milliseconds */
 
-	if (!self->priv->peaks)
+	if (self->priv->peaks == NULL || self->priv->peaks->len == 0)
 		return FALSE;
 
-	type = gdk_event_get_event_type ((GdkEvent*)event);
-	gdk_event_get_state ((GdkEvent*) event, &state);
-	gdk_event_get_button ((GdkEvent*) event, &button);
-	gdk_event_get_coords ((GdkEvent*) event, &x, NULL);
+	if (!gtk_get_current_event_state (&state))
+		return FALSE;
+
+	button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
 	clicked = (gint) x;
 	pos = pixel_to_time (self, clicked);
 
-
 	/* Single left click, no other keys pressed: new selection or changing selection */
-	if (type == GDK_BUTTON_PRESS
+	if (n_press == 1
 	    && button == GDK_BUTTON_PRIMARY
 	    && !(state & ALL_ACCELS_MASK)) {
 		/* set position as start and end point, for new selection */
@@ -477,13 +489,13 @@ pt_waveviewer_button_press_event (GtkWidget      *widget,
 			self->priv->dragend = self->priv->sel_end;
 		}
 
-		set_cursor (widget, self->priv->arrows);
+		set_cursor (GTK_WIDGET (self), self->priv->arrows);
 		update_selection (self);
 		return TRUE;
 	}
 
 	/* Single left click with Shift pressed and existing selection: enlarge selection */
-	if (type == GDK_BUTTON_PRESS
+	if (n_press == 1
 	    && button == GDK_BUTTON_PRIMARY
 	    && (state & ALL_ACCELS_MASK) == GDK_SHIFT_MASK
 	    && self->priv->sel_start != self->priv->sel_end) {
@@ -495,13 +507,13 @@ pt_waveviewer_button_press_event (GtkWidget      *widget,
 		else
 			self->priv->dragstart = self->priv->sel_start;
 
-		set_cursor (widget, self->priv->arrows);
+		set_cursor (GTK_WIDGET (self), self->priv->arrows);
 		update_selection (self);
 		return TRUE;
 	}
 
 	/* Single right click: change cursor */
-	if (type == GDK_BUTTON_PRESS && button == GDK_BUTTON_SECONDARY) {
+	if (n_press == 1 && button == GDK_BUTTON_SECONDARY) {
 		g_signal_emit_by_name (self, "cursor-changed", pos);
 		return TRUE;
 	}
@@ -529,7 +541,7 @@ pt_waveviewer_scroll_event (GtkWidget      *widget,
 		return TRUE;
 	}
 
-	/* Only Control pressed: zoom in or out */
+	/* Only Control pressed: zoom in or out TODO handle this internally without signals*/
 	if ((state & ALL_ACCELS_MASK) == GDK_CONTROL_MASK) {
 		if (delta_y < 0 || delta_x < 0) {
 			g_signal_emit_by_name (self, "zoom-out");
@@ -546,20 +558,34 @@ pt_waveviewer_scroll_event (GtkWidget      *widget,
 }
 
 static gboolean
+#if GTK_CHECK_VERSION(3,24,0)
+pt_waveviewer_motion_event (GtkEventControllerMotion *ctrl,
+                            gdouble                   x,
+			    gdouble                   y,
+			    gpointer                  user_data)
+{
+	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
+	GdkModifierType      state;
+
+	gtk_get_current_event_state (&state);
+
+#else
 pt_waveviewer_motion_notify_event (GtkWidget      *widget,
                                    GdkEventMotion *event)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (widget);
 	GdkModifierType      state;
 	gdouble              x;
-	gint64               clicked;	/* the sample clicked on */
-	gint64               pos;	/* clicked sample’s position in milliseconds */
-
-	if (!self->priv->peaks)
-		return FALSE;
 
 	gdk_event_get_state ((GdkEvent*) event, &state);
 	gdk_event_get_coords ((GdkEvent*) event, &x, NULL);
+#endif
+	gint64               clicked;	/* the sample clicked on */
+	gint64               pos;	/* clicked sample’s position in milliseconds */
+
+	if (self->priv->peaks == NULL || self->priv->peaks->len == 0)
+		return FALSE;
+
 	clicked = (gint) x;
 	pos = pixel_to_time (self, clicked);
 
@@ -580,9 +606,9 @@ pt_waveviewer_motion_notify_event (GtkWidget      *widget,
 	if (self->priv->sel_start != self->priv->sel_end) {
 		if (pointer_in_range (self, x, self->priv->sel_start)
 		    || pointer_in_range (self, x, self->priv->sel_end)) {
-			set_cursor (widget, self->priv->arrows);
+			set_cursor (GTK_WIDGET (self), self->priv->arrows);
 		} else {
-			set_cursor (widget, NULL);
+			set_cursor (GTK_WIDGET (self), NULL);
 		}
 	}
 
@@ -590,14 +616,17 @@ pt_waveviewer_motion_notify_event (GtkWidget      *widget,
 }
 
 static gboolean
-pt_waveviewer_button_release_event (GtkWidget      *widget,
-                                    GdkEventButton *event)
+pt_waveviewer_button_release_event (GtkGestureMultiPress *gesture,
+                                    gint                  n_press,
+                                    gdouble               x,
+                                    gdouble               y,
+                                    gpointer              user_data)
 {
 	guint button;
 
-	gdk_event_get_button ((GdkEvent*) event, &button);
-	if (button == GDK_BUTTON_PRIMARY) {
-		set_cursor (widget, NULL);
+	button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
+	if (n_press == 1 && button == GDK_BUTTON_PRIMARY) {
+		set_cursor (GTK_WIDGET (user_data), NULL);
 		return TRUE;
 	}
 	return FALSE;
@@ -747,7 +776,7 @@ pt_waveviewer_focus_in_event (GtkWidget *widget,
                               gpointer   data)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (widget);
-	if (!self->priv->focus_on_cursor && self->priv->peaks)
+	if (!self->priv->focus_on_cursor && self->priv->peaks->len > 0)
 		focus_widget (self);
 	return FALSE;
 }
@@ -764,7 +793,7 @@ pt_waveviewer_focus (GtkWidget        *widget,
 	   If moving into @direction would stay inside widget: return TRUE */
 
 	/* Don’t focus if empty */
-	if (!self->priv->peaks)
+	if (self->priv->peaks == NULL || self->priv->peaks->len == 0)
 		return FALSE;
 
 	/* Focus chain forward: no-focus -> focus-whole-widget -> focus-cursor -> no-focus */
@@ -805,6 +834,8 @@ pt_waveviewer_focus (GtkWidget        *widget,
  * Get follow-cursor option.
  *
  * Return value: TRUE if cursor is followed, else FALSE
+ *
+ * Since: 1.5
  */
 gboolean
 pt_waveviewer_get_follow_cursor (PtWaveviewer *self)
@@ -820,6 +851,8 @@ pt_waveviewer_get_follow_cursor (PtWaveviewer *self)
  * @follow: new value
  *
  * Set follow-cursor option to TRUE or FALSE. See also #PtWaveviewer:follow-cursor.
+ *
+ * Since: 1.5
  */
 void
 pt_waveviewer_set_follow_cursor (PtWaveviewer *self,
@@ -863,7 +896,7 @@ size_allocate_cb (GtkWidget    *widget,
 		self->priv->zoom = FALSE;
 	}
 
-	if (self->priv->peaks) {
+	if (self->priv->peaks->len > 0) {
 		pt_waveviewer_cursor_render (
 				PT_WAVEVIEWER_CURSOR (self->priv->cursor),
 				time_to_pixel (self, self->priv->playback_cursor));
@@ -875,6 +908,27 @@ size_allocate_cb (GtkWidget    *widget,
 			time_to_pixel (self, self->priv->sel_end));
 }
 
+static void
+get_anchor_point (PtWaveviewer *self)
+{
+	/* Get an anchor point = zoom_pos, either cursor position or
+	   middle of view. The anchor point is relative to the view.
+	   Get time of that anchor point = zoom_time. */
+
+	gint left;
+
+	left = (gint) gtk_adjustment_get_value (self->priv->adj);
+	if (cursor_is_visible (self)) {
+		self->priv->zoom_time = self->priv->playback_cursor;
+		self->priv->zoom_pos = time_to_pixel (self, self->priv->playback_cursor) - left;
+	} else {
+		self->priv->zoom_pos = (gint) gtk_adjustment_get_page_size (self->priv->adj) / 2;
+		self->priv->zoom_time = pixel_to_time (self, self->priv->zoom_pos + left);
+	}
+}
+
+
+#if 0
 static void
 make_widget_ready (PtWaveviewer *self,
                    gboolean      ready)
@@ -914,7 +968,7 @@ make_widget_ready (PtWaveviewer *self,
 
 	if (!ready) {
 		if (self->priv->peaks) {
-			g_free (self->priv->peaks);
+			g_array_unref (self->priv->peaks);
 			self->priv->peaks = NULL;
 		}
 		self->priv->peaks_size = 0;
@@ -936,11 +990,6 @@ make_widget_ready (PtWaveviewer *self,
 				WAVE_MIN_HEIGHT);
 	}
 
-	pt_waveviewer_waveform_set (
-			PT_WAVEVIEWER_WAVEFORM (self->priv->waveform),
-			self->priv->peaks,
-			self->priv->peaks_size);
-
 	pt_waveviewer_ruler_set_ruler (
 			PT_WAVEVIEWER_RULER (self->priv->ruler),
 			self->priv->peaks_size / 2,
@@ -954,30 +1003,6 @@ make_widget_ready (PtWaveviewer *self,
 	   above, size_allocate_cb(). */
 }
 
-static gboolean
-copy_wavedata (PtWaveviewer *self,
-               PtWavedata   *data)
-{
-	/* Copy array. If widget is not owner of its data, bad things can happen
-	   with bindings. */
-	if (!data)
-		return FALSE;
-
-	if (!data->array || !data->length)
-		return FALSE;
-
-	if (!(self->priv->peaks = g_malloc (sizeof (gfloat) * data->length))) {
-		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
-			          "MESSAGE", "Waveviewer failed to allocate memory");
-		return FALSE;
-	}
-
-	memcpy (self->priv->peaks, data->array, sizeof(gfloat) * data->length);
-	self->priv->peaks_size = data->length;
-	self->priv->px_per_sec = data->px_per_sec;
-	return TRUE;
-}
-
 /**
  * pt_waveviewer_set_wave:
  * @self: the widget
@@ -989,27 +1014,238 @@ copy_wavedata (PtWaveviewer *self,
  */
 void
 pt_waveviewer_set_wave (PtWaveviewer *self,
-                        PtWavedata   *data)
+                        GArray       *data)
 {
 	g_return_if_fail (PT_IS_WAVEVIEWER (self));
 
 	make_widget_ready (self, FALSE);
-	if (!copy_wavedata (self, data))
+	if (data == NULL || data->len == 0)
 		return;
+
+	self->priv->peaks = g_array_ref (data);
+	self->priv->peaks_size = data->len;
+	self->priv->px_per_sec = self->priv->pps;
+
 	make_widget_ready (self, TRUE);
 }
+#endif
+
+static void
+array_size_changed_cb (PtWaveloader *loader,
+		       gpointer      user_data)
+{
+	PtWaveviewer *self = PT_WAVEVIEWER (user_data);
+	gint widget_width;
+
+	if (self->priv->peaks == NULL || self->priv->peaks->len == 0) {
+		self->priv->peaks_size = 0;
+		self->priv->px_per_sec = 0;
+		self->priv->duration = 0;
+		pt_waveviewer_selection_set (
+				PT_WAVEVIEWER_SELECTION (self->priv->selection),
+				0, 0);
+		pt_waveviewer_cursor_render (
+				PT_WAVEVIEWER_CURSOR (self->priv->cursor),
+				-1);
+	} else {
+		self->priv->peaks_size = self->priv->peaks->len;
+		self->priv->px_per_sec = self->priv->pps;
+		self->priv->duration = calculate_duration (self);
+		widget_width = self->priv->peaks_size / 2;
+		gtk_adjustment_set_upper (self->priv->adj, widget_width);
+		gtk_widget_set_size_request (
+				self->priv->waveform,
+				widget_width,
+				WAVE_MIN_HEIGHT);
+	}
+
+	pt_waveviewer_ruler_set_ruler (
+			PT_WAVEVIEWER_RULER (self->priv->ruler),
+			self->priv->peaks_size / 2,
+			self->priv->px_per_sec,
+			self->priv->duration);
+}
+
+static void
+propagate_progress_cb (PtWaveloader *loader,
+                       gdouble       progress,
+                       PtWaveviewer *self)
+{
+	g_signal_emit_by_name (self, "load-progress", progress);
+}
+
+static void
+load_cb (PtWaveloader *loader,
+         GAsyncResult *res,
+         gpointer     *data)
+{
+	GTask        *task = (GTask *) data;
+	PtWaveviewer *self = g_task_get_source_object (task);
+	GError       *error = NULL;
+
+	if (self->priv->tick_handler != 0) {
+		gtk_widget_remove_tick_callback (self->priv->waveform,
+						 self->priv->tick_handler);
+		self->priv->tick_handler = 0;
+	}
+
+	if (pt_waveloader_load_finish (loader, res, &error)) {
+		array_size_changed_cb (NULL, self);
+		gtk_widget_queue_draw (self->priv->waveform);
+		g_task_return_boolean (task, TRUE);
+	} else {
+		g_array_set_size (self->priv->peaks, 0);
+		array_size_changed_cb (NULL, self);
+		g_task_return_error (task, error);
+	}
+
+	g_object_unref (task);
+}
+
+/**
+ * pt_waveviewer_load_wave_finish:
+ * @self: the widget
+ * @result: the #GAsyncResult passed to your #GAsyncReadyCallback
+ * @error: (nullable): a pointer to a NULL #GError, or NULL
+ *
+ * Gives the result of the async load operation. A cancelled operation results
+ * in an error, too.
+ *
+ * Return value: TRUE if successful, or FALSE with error set
+ *
+ * Since: 2.0
+ */
+gboolean
+pt_waveviewer_load_wave_finish (PtWaveviewer  *self,
+                                GAsyncResult  *result,
+                                GError       **error)
+{
+	g_return_val_if_fail (g_task_is_valid (result, self), FALSE);
+
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
+static gboolean
+update_waveform_cb (GtkWidget     *widget,
+                    GdkFrameClock *frame_clock,
+                    gpointer       data)
+{
+	gtk_widget_queue_draw (widget);
+	return G_SOURCE_CONTINUE;
+}
+
+/**
+ * pt_waveviewer_load_wave_async:
+ * @self: the widget
+ * @uri: the URI of the file
+ * @cancel: (nullable): a #GCancellable or NULL
+ * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
+ * @user_data: (closure): user data for callback
+ *
+ * Load wave form for the given URI. The initial resolution is set to
+ * #PtWaveviewer:pps. While loading, a #PtWaveviewer::load-progress signal is
+ * emitted. A previous waveform is discarded.
+ *
+ * Since: 2.0
+ */
+void
+pt_waveviewer_load_wave_async (PtWaveviewer        *self,
+                               gchar               *uri,
+                               GCancellable        *cancel,
+                               GAsyncReadyCallback  callback,
+                               gpointer             user_data)
+{
+	g_return_if_fail (PT_IS_WAVEVIEWER (self));
+	g_return_if_fail (uri != NULL);
+
+	GTask  *task;
+	/* TODO think about doing this here or just returning error from waveloader.
+	 * this is meant for "nicer" error messages, requires translation
+	GFile  *file;
+	gchar  *location = NULL;*/
+
+	task = g_task_new (self, NULL, callback, user_data);
+
+#if 0
+	/* Change uri to location */
+	file = g_file_new_for_uri (uri);
+	location = g_file_get_path (file);
+
+	if (!location) {
+		g_task_return_new_error (
+				task,
+				GST_RESOURCE_ERROR,
+				GST_RESOURCE_ERROR_NOT_FOUND,
+				/* Translators: %s is a detailed error message. */
+				_("URI not valid: %s"), uri);
+		g_object_unref (file);
+		g_object_unref (task);
+		return;
+	}
+
+	if (!g_file_query_exists (file, NULL)) {
+		g_task_return_new_error (
+				task,
+				GST_RESOURCE_ERROR,
+				GST_RESOURCE_ERROR_NOT_FOUND,
+				/* Translators: %s is a detailed error message. */
+				_("File not found: %s"), location);
+		g_object_unref (file);
+		g_free (location);
+		g_object_unref (task);
+		return;
+	}
+
+	g_object_unref (file);
+	g_free (location);
+#endif
+
+	g_object_set (self->priv->loader, "uri", uri, NULL);
+	self->priv->px_per_sec = self->priv->pps;
+	if (self->priv->tick_handler == 0) {
+		self->priv->tick_handler =
+			gtk_widget_add_tick_callback (
+				self->priv->waveform,
+				(GtkTickCallback) update_waveform_cb,
+				self, NULL);
+	}
+	pt_waveloader_load_async (self->priv->loader,
+				  self->priv->pps,
+				  cancel,
+				  (GAsyncReadyCallback) load_cb,
+				  task);
+}
 
 static void
 pt_waveviewer_finalize (GObject *object)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (object);
 
-	g_free (self->priv->peaks);
+	g_array_unref (self->priv->peaks);
 	g_clear_object (&self->priv->arrows);
+	g_clear_object (&self->priv->loader);
+	if (self->priv->tick_handler != 0) {
+		gtk_widget_remove_tick_callback (self->priv->waveform,
+						 self->priv->tick_handler);
+		self->priv->tick_handler = 0;
+	}
 
 	G_OBJECT_CLASS (pt_waveviewer_parent_class)->finalize (object);
 }
 
+static void
+pt_waveviewer_dispose (GObject *object)
+{
+	PtWaveviewer *self = PT_WAVEVIEWER (object);
+
+	g_clear_object (&self->priv->button);
+#if GTK_CHECK_VERSION(3,24,0)
+	g_clear_object (&self->priv->motion_ctrl);
+#endif
+
+	G_OBJECT_CLASS (pt_waveviewer_parent_class)->dispose (object);
+}
+
 static void
 pt_waveviewer_get_property (GObject    *object,
                             guint       property_id,
@@ -1037,6 +1273,9 @@ pt_waveviewer_get_property (GObject    *object,
 	case PROP_SELECTION_END:
 		g_value_set_int64 (value, self->priv->sel_end);
 		break;
+	case PROP_PPS:
+		g_value_set_int (value, self->priv->pps);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -1050,6 +1289,7 @@ pt_waveviewer_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
 	PtWaveviewer *self = PT_WAVEVIEWER (object);
+	GError *error = NULL;
 
 	switch (property_id) {
 	case PROP_PLAYBACK_CURSOR:
@@ -1082,8 +1322,18 @@ pt_waveviewer_set_property (GObject      *object,
 		gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->revealer),
 					       self->priv->show_ruler);
 		break;
-	case PROP_ZOOM:
-		self->priv->zoom = g_value_get_boolean (value);
+	case PROP_PPS:
+		self->priv->pps = g_value_get_int (value);
+		if (self->priv->peaks->len == 0)
+			break;
+		get_anchor_point (self);
+		self->priv->zoom = TRUE;
+		if (!pt_waveloader_resize (self->priv->loader,
+				           self->priv->pps,
+				           &error)) {
+			g_print ("%s\n", error->message);
+			g_clear_error (&error);
+		}
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -1130,7 +1380,6 @@ pt_waveviewer_init (PtWaveviewer *self)
 	GtkCssProvider  *provider;
 	GFile		*css_file;
 
-	self->priv->peaks = NULL;
 	self->priv->peaks_size = 0;
 	self->priv->duration = 0;
 	self->priv->playback_cursor = 0;
@@ -1147,6 +1396,13 @@ pt_waveviewer_init (PtWaveviewer *self)
 	self->priv->selection = pt_waveviewer_selection_new ();
 	self->priv->ruler = pt_waveviewer_ruler_new ();
 	self->priv->revealer = gtk_revealer_new ();
+	self->priv->loader = pt_waveloader_new (NULL);
+	self->priv->peaks = pt_waveloader_get_data (self->priv->loader);
+	self->priv->tick_handler = 0;
+
+	pt_waveviewer_waveform_set (
+			PT_WAVEVIEWER_WAVEFORM (self->priv->waveform),
+			self->priv->peaks);
 
 	/* Setup scrolled window */
 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -1173,7 +1429,7 @@ pt_waveviewer_init (PtWaveviewer *self)
 	gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (self), FALSE);
 	gtk_widget_show_all (GTK_WIDGET (self));
 
-	css_file = g_file_new_for_uri ("resource:///com/github/gkarsay/libparlatype/pt-waveviewer.css");
+	css_file = g_file_new_for_uri ("resource:///org/parlatype/libparlatype/pt-waveviewer.css");
 	provider = gtk_css_provider_new ();
 	gtk_css_provider_load_from_file (provider, css_file, NULL);
 	gtk_style_context_add_provider_for_screen (
@@ -1190,11 +1446,47 @@ pt_waveviewer_init (PtWaveviewer *self)
 			G_CALLBACK (size_allocate_cb),
 			self);
 
+	/* Setup event handling */
+	self->priv->button = gtk_gesture_multi_press_new (box);
+	gtk_gesture_single_set_exclusive (GTK_GESTURE_SINGLE (self->priv->button), TRUE);
+	gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (self->priv->button), 0);
+	gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->priv->button), GTK_PHASE_CAPTURE);
+	g_signal_connect (
+			self->priv->button,
+			"pressed",
+			G_CALLBACK (pt_waveviewer_button_press_event),
+			self);
+	g_signal_connect (
+			self->priv->button,
+			"released",
+			G_CALLBACK (pt_waveviewer_button_release_event),
+			self);
+
+#if GTK_CHECK_VERSION(3,24,0)
+	self->priv->motion_ctrl = gtk_event_controller_motion_new (box);
+	g_signal_connect (
+			self->priv->motion_ctrl,
+			"motion",
+			G_CALLBACK (pt_waveviewer_motion_event),
+			self);
+#endif
+
 	/* If overriding these vfuncs something’s going wrong, note that focus-in
 	   an focus-out need GdkEventFocus as 2nd parameter in vfunc */
 	g_signal_connect (self, "focus", G_CALLBACK (pt_waveviewer_focus), NULL);
 	g_signal_connect (self, "focus-in-event", G_CALLBACK (pt_waveviewer_focus_in_event), NULL);
 	g_signal_connect (self, "focus-out-event", G_CALLBACK (pt_waveviewer_focus_out_event), NULL);
+
+	g_signal_connect (self->priv->loader,
+			"progress",
+			G_CALLBACK (propagate_progress_cb),
+			self);
+
+	g_signal_connect (self->priv->loader,
+			"array-size-changed",
+			G_CALLBACK (array_size_changed_cb),
+			self);
+
 }
 
 static void
@@ -1206,18 +1498,39 @@ pt_waveviewer_class_init (PtWaveviewerClass *klass)
 	gobject_class->set_property = pt_waveviewer_set_property;
 	gobject_class->get_property = pt_waveviewer_get_property;
 	gobject_class->constructed  = pt_waveviewer_constructed;
+	gobject_class->dispose      = pt_waveviewer_dispose;
 	gobject_class->finalize     = pt_waveviewer_finalize;
 
-	widget_class->button_press_event   = pt_waveviewer_button_press_event;
-	widget_class->button_release_event = pt_waveviewer_button_release_event;
 	widget_class->direction_changed    = pt_waveviewer_direction_changed;
 	widget_class->key_press_event      = pt_waveviewer_key_press_event;
+#if GTK_CHECK_VERSION(3,24,0)
+#else
 	widget_class->motion_notify_event  = pt_waveviewer_motion_notify_event;
+#endif
 	widget_class->realize              = pt_waveviewer_realize;
 	widget_class->scroll_event         = pt_waveviewer_scroll_event;
 	widget_class->state_flags_changed  = pt_waveviewer_state_flags_changed;
 	widget_class->style_updated        = pt_waveviewer_style_updated;
 
+	/**
+	* PtWaveviewer::load-progress:
+	* @viewer: the waveviewer emitting the signal
+	* @progress: the new progress state, ranging from 0.0 to 1.0
+	*
+	* Indicates progress on a scale from 0.0 to 1.0, however it does not
+	* emit the value 0.0 nor 1.0. Wait for a TRUE player-state-changed
+	* signal or an error signal to dismiss a gui element showing progress.
+	*/
+	g_signal_new ("load-progress",
+		      PT_TYPE_WAVEVIEWER,
+		      G_SIGNAL_RUN_FIRST,
+		      0,
+		      NULL,
+		      NULL,
+		      g_cclosure_marshal_VOID__DOUBLE,
+		      G_TYPE_NONE,
+		      1, G_TYPE_DOUBLE);
+
 	/**
 	* PtWaveviewer::cursor-changed:
 	* @viewer: the waveviewer emitting the signal
@@ -1438,21 +1751,19 @@ pt_waveviewer_class_init (PtWaveviewerClass *klass)
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	/**
-	* PtWaveviewer:zoom:
+	* PtWaveviewer:pps:
 	*
-	* Set this to TRUE to indicate that the next wave data will be the same
-	* waveform but at a different zoom level. PtWaveviewer needs this to
-	* decide which part of the waveform should be shown.
-	* – Note: This will be probably removed with the next API break in
-	* version 1.6 and integrated into pt_waveviewer_set_wave().
+	* Current/requested resolution of waveform in pixels per second.
 	*/
 
-	obj_properties[PROP_ZOOM] =
-	g_param_spec_boolean (
-			"zoom",
-			"Zoom",
-			"Indicates that next wave data will zoom",
-			FALSE,
+	obj_properties[PROP_PPS] =
+	g_param_spec_int (
+			"pps",
+			"Pixels per second",
+			"Current/requested resolution of waveform in pixels per second",
+			25,
+			200,
+			100,
 			G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties (
@@ -1464,12 +1775,13 @@ pt_waveviewer_class_init (PtWaveviewerClass *klass)
 /**
  * pt_waveviewer_new:
  *
- * Create a new, initially blank waveform viewer widget. Use
- * pt_waveviewer_set_wave() to pass wave data.
+ * Create a new, initially blank waveform viewer widget.
  *
  * After use gtk_widget_destroy() it.
  *
  * Returns: the widget
+ *
+ * Since: 1.5
  */
 GtkWidget *
 pt_waveviewer_new (void)
