@@ -18,6 +18,16 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>	
+#ifdef G_OS_UNIX
+  #include "pt-dbus-service.h"
+  #include "pt-mediakeys.h"
+#endif
+#ifdef G_OS_WIN32
+  #include "pt-win32-datacopy.h"
+  #include "pt-win32-helpers.h"
+  #include "pt-win32-hotkeys.h"
+  #include "pt-win32-pipe.h"
+#endif
 #include "pt-preferences.h"
 #include "pt-window.h"
 #include "pt-app.h"
@@ -27,6 +37,15 @@ struct _PtAppPrivate
 	gboolean       asr;
 	gboolean       atspi;
 	PtAsrSettings *asr_settings;
+#ifdef G_OS_UNIX
+	PtMediakeys   *mediakeys;
+	PtDbusService *dbus_service;
+#endif
+#ifdef G_OS_WIN32
+	PtWin32Datacopy *win32datacopy;
+	PtWin32Hotkeys  *win32hotkeys;
+	PtWin32Pipe     *win32pipe;
+#endif
 };
 
 
@@ -105,7 +124,11 @@ open_cb (GSimpleAction *action,
 	filter_all = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter_audio, _("Audio files"));
 	gtk_file_filter_set_name (filter_all, _("All files"));
+#ifdef G_OS_WIN32
+	pt_win32_add_audio_patterns (filter_audio);
+#else
 	gtk_file_filter_add_mime_type (filter_audio, "audio/*");
+#endif
 	gtk_file_filter_add_pattern (filter_all, "*");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter_audio);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter_all);
@@ -140,14 +163,21 @@ help_cb (GSimpleAction *action,
          gpointer       app)
 {
 	GtkWindow *win;
+	gchar     *uri;
+	GError    *error = NULL;
+	gchar     *errmsg;
+
 	win = gtk_application_get_active_window (app);
 
-	GError *error = NULL;
-	gchar  *errmsg;
+#ifdef G_OS_WIN32
+	uri = pt_win32_get_help_uri ();
+#else
+	uri = g_strdup_printf ("help:%s", APP_ID);
+#endif
 
 	gtk_show_uri_on_window (
 			win,
-			"help:com.github.gkarsay.parlatype",
+			uri,
 			GDK_CURRENT_TIME,
 			&error);
 
@@ -158,6 +188,8 @@ help_cb (GSimpleAction *action,
 		g_free (errmsg);
 		g_error_free (error);
 	}
+
+	g_free (uri);
 }
 
 static void
@@ -188,9 +220,9 @@ about_cb (GSimpleAction *action,
 			gtk_application_get_active_window (app),
 			"program_name", _("Parlatype"),
 			"version", PACKAGE_VERSION,
-			"copyright", "© Gabor Karsay 2016–2019",
+			"copyright", "© Gabor Karsay 2016–2020",
 			"comments", _("A basic transcription utility"),
-			"logo-icon-name", "com.github.gkarsay.parlatype",
+			"logo-icon-name", APP_ID,
 			"authors", authors,
 			"artists", artists,
 			"translator-credits", _("translator-credits"),
@@ -239,7 +271,7 @@ pt_app_get_asr (PtApp *app)
 static void
 pt_app_startup (GApplication *app)
 {
-	g_application_set_resource_base_path (app, "/com/github/gkarsay/parlatype");
+	g_application_set_resource_base_path (app, "/org/parlatype/parlatype");
 	G_APPLICATION_CLASS (pt_app_parent_class)->startup (app);
 
 	g_action_map_add_action_entries (G_ACTION_MAP (app),
@@ -282,7 +314,7 @@ pt_app_startup (GApplication *app)
 	/* Load custom css */
 	GtkCssProvider *provider;
 	provider = gtk_css_provider_new ();
-	gtk_css_provider_load_from_resource (provider, "/com/github/gkarsay/parlatype/parlatype.css");
+	gtk_css_provider_load_from_resource (provider, "/org/parlatype/parlatype/parlatype.css");
 	gtk_style_context_add_provider_for_screen (
 			gdk_screen_get_default (),
 			GTK_STYLE_PROVIDER (provider),
@@ -291,16 +323,37 @@ pt_app_startup (GApplication *app)
 }
 
 static void
-pt_app_activate (GApplication *app)
+pt_app_activate (GApplication *application)
 {
-	GList *windows;
+	PtApp    *app = PT_APP (application);
+	GList    *windows;
 	PtWindow *win;
 
-	windows = gtk_application_get_windows (GTK_APPLICATION (app));
-	if (windows)
+#ifdef G_OS_WIN32
+	if (pt_win32_present_other_instance (application))
+		g_application_quit (application);
+#endif
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (application));
+	if (windows) {
 		win = PT_WINDOW (windows->data);
-	else
-		win = pt_window_new (PT_APP (app));
+	} else {
+		win = pt_window_new (app);
+#ifdef G_OS_UNIX
+		app->priv->mediakeys = pt_mediakeys_new (win);
+		pt_mediakeys_start (app->priv->mediakeys);
+		app->priv->dbus_service = pt_dbus_service_new (win);
+		pt_dbus_service_start (app->priv->dbus_service);
+#endif
+#ifdef G_OS_WIN32
+		app->priv->win32datacopy = pt_win32_datacopy_new (win);
+		pt_win32_datacopy_start (app->priv->win32datacopy);
+		app->priv->win32hotkeys = pt_win32_hotkeys_new (win);
+		pt_win32_hotkeys_start (app->priv->win32hotkeys);
+		app->priv->win32pipe = pt_win32_pipe_new (win);
+		pt_win32_pipe_start (app->priv->win32pipe);
+#endif
+	}
 
 	gtk_window_present (GTK_WINDOW (win));
 }
@@ -314,14 +367,21 @@ pt_app_open (GApplication  *app,
 	GtkWindow *win;
 	gchar	  *uri;
 
-	pt_app_activate (app);
-	win = gtk_application_get_active_window (GTK_APPLICATION (app));
-
 	if (n_files > 1) {
 		g_print (_("Warning: Parlatype handles only one file at a time. The other files are ignored.\n"));
 	}
 
 	uri = g_file_get_uri (files[0]);
+
+#ifdef G_OS_WIN32
+	if (pt_win32_open_in_other_instance (app, uri)) {
+		g_free (uri);
+		g_application_quit (app);
+	}
+#endif
+
+	pt_app_activate (app);
+	win = gtk_application_get_active_window (GTK_APPLICATION (app));
 	pt_window_open_file (PT_WINDOW (win), uri);
 	g_free (uri);
 }
@@ -383,6 +443,15 @@ pt_app_init (PtApp *app)
 {
 	app->priv = pt_app_get_instance_private (app);
 
+#ifdef G_OS_UNIX
+	app->priv->mediakeys = NULL;
+	app->priv->dbus_service = NULL;
+#endif
+#ifdef G_OS_WIN32
+	app->priv->win32datacopy = NULL;
+	app->priv->win32hotkeys = NULL;
+	app->priv->win32pipe = NULL;
+#endif
 	g_application_add_main_option_entries (G_APPLICATION (app), options);
 
 	/* In Flatpak's sandbox ATSPI doesn't work */
@@ -404,6 +473,15 @@ pt_app_finalize (GObject *object)
 	PtApp *app = PT_APP (object);
 
 	g_clear_object (&app->priv->asr_settings);
+#ifdef G_OS_UNIX
+	g_clear_object (&app->priv->mediakeys);
+	g_clear_object (&app->priv->dbus_service);
+#endif
+#ifdef G_OS_WIN32
+	g_clear_object (&app->priv->win32datacopy);
+	g_clear_object (&app->priv->win32hotkeys);
+	g_clear_object (&app->priv->win32pipe);
+#endif
 
 	G_OBJECT_CLASS (pt_app_parent_class)->dispose (object);
 }
@@ -425,7 +503,7 @@ PtApp *
 pt_app_new (void)
 {
 	return g_object_new (PT_APP_TYPE,
-			     "application-id", "com.github.gkarsay.parlatype",
+			     "application-id", APP_ID,
 			     "flags", G_APPLICATION_HANDLES_OPEN,
 			     NULL);
 }
