@@ -85,7 +85,7 @@ struct _PtPreferencesDialogPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (PtPreferencesDialog, pt_preferences_dialog, GTK_TYPE_DIALOG)
 
-void
+static void
 spin_back_changed_cb (GtkSpinButton       *spin,
                       PtPreferencesDialog *dlg)
 {
@@ -99,7 +99,7 @@ spin_back_changed_cb (GtkSpinButton       *spin,
 				  (int) gtk_spin_button_get_value_as_int (spin)));
 }
 
-void
+static void
 spin_forward_changed_cb (GtkSpinButton       *spin,
                          PtPreferencesDialog *dlg)
 {
@@ -109,7 +109,7 @@ spin_forward_changed_cb (GtkSpinButton       *spin,
 				  (int) gtk_spin_button_get_value_as_int (spin)));
 }
 
-void
+static void
 spin_pause_changed_cb (GtkSpinButton       *spin,
                        PtPreferencesDialog *dlg)
 {
@@ -119,38 +119,6 @@ spin_pause_changed_cb (GtkSpinButton       *spin,
 				  (int) gtk_spin_button_get_value_as_int (spin)));
 }
 
-gchar*
-format_value_cb (GtkScale *scale,
-                 gdouble   value,
-                 gpointer  data)
-{
-	return g_strdup_printf ("%d", 25 * (gint) value);
-}
-
-gboolean
-get_pps (GValue   *value,
-         GVariant *variant,
-         gpointer  data)
-{
-	gint32 pps;
-	pps = g_variant_get_int32 (variant);
-	if (pps < 25)
-		pps = 25;
-	g_value_set_double (value, (gdouble) (pps / 25));
-	return TRUE;
-}
-
-GVariant*
-set_pps (const GValue       *value,
-         const GVariantType *type,
-         gpointer            data)
-{
-	gdouble pps;
-	pps = g_value_get_double (value);
-	pps = pps * 25;
-	return g_variant_new_int32 ((gint32) pps);
-}
-
 static void
 update_example_timestamps (PtPreferencesDialog *dlg)
 {
@@ -165,7 +133,7 @@ update_example_timestamps (PtPreferencesDialog *dlg)
 	g_free (example2);
 }
 
-void
+static void
 hours_check_toggled (GtkToggleButton     *check,
                      PtPreferencesDialog *dlg)
 {
@@ -175,7 +143,7 @@ hours_check_toggled (GtkToggleButton     *check,
 	update_example_timestamps (dlg);
 }
 
-void
+static void
 delimiter_combo_changed (GtkComboBox         *widget,
                          PtPreferencesDialog *dlg)
 {
@@ -185,7 +153,7 @@ delimiter_combo_changed (GtkComboBox         *widget,
 	update_example_timestamps (dlg);
 }
 
-void
+static void
 separator_combo_changed (GtkComboBox         *widget,
                          PtPreferencesDialog *dlg)
 {
@@ -195,7 +163,7 @@ separator_combo_changed (GtkComboBox         *widget,
 	update_example_timestamps (dlg);
 }
 
-void
+static void
 precision_combo_changed (GtkComboBox         *widget,
                          PtPreferencesDialog *dlg)
 {
@@ -262,14 +230,14 @@ launch_asr_assistant (PtPreferencesDialog *dlg)
 	g_object_unref (parent);
 }
 
-void
+static void
 add_asr_button_clicked_cb (GtkToolButton       *button,
                            PtPreferencesDialog *dlg)
 {
 	launch_asr_assistant (dlg);
 }
 
-void
+static void
 initial_asr_button_clicked_cb (GtkButton           *button,
                                PtPreferencesDialog *dlg)
 {
@@ -304,7 +272,7 @@ confirm_delete (GtkWindow *parent,
 	return (response == GTK_RESPONSE_YES);
 }
 
-void
+static void
 remove_asr_button_clicked_cb (GtkToolButton       *button,
                               PtPreferencesDialog *dlg)
 {
@@ -383,7 +351,7 @@ set_active (PtPreferencesDialog *dlg,
 	g_free (new);
 }
 
-void
+static void
 asr_default_toggled_cb (GtkCellRendererToggle *cell,
                         gchar                 *path_string,
                         gpointer               user_data)
@@ -438,7 +406,7 @@ get_selected_id (GtkTreeView *view)
 	return id;
 }
 
-void
+static void
 prefs_lm_chooser_file_set_cb (GtkFileChooserButton *button,
                               PtPreferencesDialog  *dlg)
 {
@@ -455,7 +423,7 @@ prefs_lm_chooser_file_set_cb (GtkFileChooserButton *button,
 	g_free (lm_uri);
 }
 
-void
+static void
 prefs_dict_chooser_file_set_cb (GtkFileChooserButton *button,
                                 PtPreferencesDialog  *dlg)
 {
@@ -472,7 +440,7 @@ prefs_dict_chooser_file_set_cb (GtkFileChooserButton *button,
 	g_free (dict_uri);
 }
 
-void
+static void
 prefs_hmm_chooser_file_set_cb (GtkFileChooserButton *button,
                                PtPreferencesDialog  *dlg)
 {
@@ -527,11 +495,25 @@ asr_selection_changed_cb (GtkTreeSelection *sel,
 	g_free (hmm_uri);
 }
 
+static void
+setup_non_wayland_env (PtPreferencesDialog *dlg)
+{
+	g_settings_bind (
+		dlg->priv->editor, "remember-position",
+		dlg->priv->pos_check, "active",
+		G_SETTINGS_BIND_DEFAULT);
+
+	g_settings_bind (
+		dlg->priv->editor, "start-on-top",
+		dlg->priv->top_check, "active",
+		G_SETTINGS_BIND_DEFAULT);
+}
+
 static void
 pt_preferences_dialog_init (PtPreferencesDialog *dlg)
 {
 	dlg->priv = pt_preferences_dialog_get_instance_private (dlg);
-	dlg->priv->editor = g_settings_new ("com.github.gkarsay.parlatype");
+	dlg->priv->editor = g_settings_new (APP_ID);
 	dlg->priv->player = pt_player_new ();
 	pt_player_setup_player (dlg->priv->player, NULL); /* no error handling, already checked in main window */
 
@@ -570,20 +552,15 @@ pt_preferences_dialog_init (PtPreferencesDialog *dlg)
 			dlg->priv->size_check, "active",
 			G_SETTINGS_BIND_DEFAULT);
 
+#ifdef GDK_WINDOWING_WIN32
+	setup_non_wayland_env (dlg);
+#else
 	GdkDisplay *display;
 	display = gdk_display_get_default ();
+#endif
 #ifdef GDK_WINDOWING_X11
-	if (GDK_IS_X11_DISPLAY (display)) {
-		g_settings_bind (
-				dlg->priv->editor, "remember-position",
-				dlg->priv->pos_check, "active",
-				G_SETTINGS_BIND_DEFAULT);
-
-		g_settings_bind (
-				dlg->priv->editor, "start-on-top",
-				dlg->priv->top_check, "active",
-				G_SETTINGS_BIND_DEFAULT);
-	}
+	if (GDK_IS_X11_DISPLAY (display))
+		setup_non_wayland_env (dlg);
 #endif
 #ifdef GDK_WINDOWING_WAYLAND
 	if (GDK_IS_WAYLAND_DISPLAY (display)) {
@@ -604,12 +581,16 @@ pt_preferences_dialog_init (PtPreferencesDialog *dlg)
 
 	GtkAdjustment *pps_adj;
 	pps_adj = gtk_range_get_adjustment (GTK_RANGE (dlg->priv->pps_scale));
-	g_settings_bind_with_mapping (
+	g_settings_bind (
 			dlg->priv->editor, "pps",
 			pps_adj, "value",
-			G_SETTINGS_BIND_DEFAULT,
-			get_pps, set_pps,
-			NULL, NULL);
+			G_SETTINGS_BIND_DEFAULT);
+	GtkScale *scale = GTK_SCALE (dlg->priv->pps_scale);
+	gtk_scale_add_mark (scale, 25, GTK_POS_TOP, NULL);
+	gtk_scale_add_mark (scale, 62.5, GTK_POS_TOP, NULL);
+	gtk_scale_add_mark (scale, 100, GTK_POS_TOP, NULL);
+	gtk_scale_add_mark (scale, 150, GTK_POS_TOP, NULL);
+	gtk_scale_add_mark (scale, 200, GTK_POS_TOP, NULL);
 
 	/* make sure labels are set and translated */
 	spin_back_changed_cb (GTK_SPIN_BUTTON (dlg->priv->spin_back), dlg);
@@ -731,7 +712,21 @@ pt_preferences_dialog_class_init (PtPreferencesDialogClass *klass)
 	object_class->dispose = pt_preferences_dialog_dispose;
 
 	/* Bind class to template */
-	gtk_widget_class_set_template_from_resource (widget_class, "/com/github/gkarsay/parlatype/preferences.ui");
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/parlatype/parlatype/preferences.ui");
+	gtk_widget_class_bind_template_callback(widget_class, add_asr_button_clicked_cb);
+	gtk_widget_class_bind_template_callback(widget_class, asr_default_toggled_cb);
+	gtk_widget_class_bind_template_callback(widget_class, delimiter_combo_changed);
+	gtk_widget_class_bind_template_callback(widget_class, hours_check_toggled);
+	gtk_widget_class_bind_template_callback(widget_class, initial_asr_button_clicked_cb);
+	gtk_widget_class_bind_template_callback(widget_class, precision_combo_changed);
+	gtk_widget_class_bind_template_callback(widget_class, prefs_hmm_chooser_file_set_cb);
+	gtk_widget_class_bind_template_callback(widget_class, prefs_dict_chooser_file_set_cb);
+	gtk_widget_class_bind_template_callback(widget_class, prefs_lm_chooser_file_set_cb);
+	gtk_widget_class_bind_template_callback(widget_class, remove_asr_button_clicked_cb);
+	gtk_widget_class_bind_template_callback(widget_class, separator_combo_changed);
+	gtk_widget_class_bind_template_callback(widget_class, spin_back_changed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, spin_forward_changed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, spin_pause_changed_cb);
 	gtk_widget_class_bind_template_child_private (widget_class, PtPreferencesDialog, notebook);
 	gtk_widget_class_bind_template_child_private (widget_class, PtPreferencesDialog, spin_pause);
 	gtk_widget_class_bind_template_child_private (widget_class, PtPreferencesDialog, spin_back);
