@@ -19,26 +19,24 @@
 
 #include "config.h"
 #include <gtk/gtk.h>
-#include <pt-asr-output.h>
 #include <pt-player.h>
-#include <pt-wavedata.h>
+#ifdef HAVE_ASR
+#include "pt-asr-output.h"
+#include "pt-asr-settings.h"
+#endif
 
 struct _PtWindowPrivate
 {
 	GSettings	 *editor;
-	PtAsrSettings    *asr_settings;
-	PtAsrOutput      *output;
-	GDBusProxy	 *proxy;
-	gint              dbus_watch_id;
 	GtkRecentManager *recent;
 	GtkAccelGroup    *accels;
-	PtPlayer	 *player;
 
-	GtkWidget  *progress_dlg;
-	gint	    progress_handler_id;
-
-	guint       output_handler_id1;
-	guint       output_handler_id2;
+#ifdef HAVE_ASR
+	PtAsrSettings    *asr_settings;
+	PtAsrOutput      *output;
+	guint             output_handler_id1;
+	guint             output_handler_id2;
+#endif
 
 	GtkClipboard *clip;
 	gulong        clip_handler_id;
@@ -48,12 +46,15 @@ struct _PtWindowPrivate
 	GtkWidget  *primary_menu_button;
 
 	/* Main window widgets */
+	GtkWidget  *progress;
 	GtkWidget  *button_play;
 	GtkWidget  *button_fast_back;	  // not used
 	GtkWidget  *button_fast_forward;  // not used
 	GtkWidget  *button_jump_back;
 	GtkWidget  *button_jump_forward;
 	GtkWidget  *volumebutton;
+	GStrv      *vol_icons;
+	GtkGesture *vol_event;
 	GtkWidget  *pos_menu_button;
 	GtkWidget  *pos_label;
 	GMenuItem  *go_to_timestamp;
@@ -61,14 +62,10 @@ struct _PtWindowPrivate
 	GtkWidget  *waveviewer;
 	GMenuModel *secondary_menu;
 
-	PtWavedata *wavedata;
-
 	gint64      last_time;	// last time to compare if it changed
 
 	gint	    timer;
 	gdouble	    speed;
-
-	guint	    owner_id;
 };
 
 #endif
