@@ -23,13 +23,10 @@
 #include <gdk/gdkx.h>
 #endif
 #include <pt-player.h>
-#include <pt-progress-dialog.h>
 #include <pt-waveviewer.h>
 #include "pt-app.h"
 #include "pt-asr-assistant.h"
-#include "pt-dbus-service.h"
 #include "pt-goto-dialog.h"
-#include "pt-mediakeys.h"
 #include "pt-window.h"
 #include "pt-window-dnd.h"
 #include "pt-window-private.h"
@@ -37,9 +34,9 @@
 
 G_DEFINE_TYPE_WITH_PRIVATE (PtWindow, pt_window, GTK_TYPE_APPLICATION_WINDOW)
 
-void play_button_toggled_cb (GtkToggleButton *button, PtWindow *win);
-void jump_back_button_clicked_cb (GtkButton *button, PtWindow *win);
-void jump_forward_button_clicked_cb (GtkButton *button, PtWindow *win);
+static void play_button_toggled_cb (GtkToggleButton *button, PtWindow *win);
+static void jump_back_button_clicked_cb (GtkButton *button, PtWindow *win);
+static void jump_forward_button_clicked_cb (GtkButton *button, PtWindow *win);
 
 void
 pt_error_message (PtWindow    *parent,
@@ -73,7 +70,7 @@ copy_timestamp (GSimpleAction *action,
 	win = PT_WINDOW (user_data);
 	const gchar  *timestamp = NULL;
 
-	timestamp = pt_player_get_timestamp (win->priv->player);
+	timestamp = pt_player_get_timestamp (win->player);
 	if (timestamp)
 		gtk_clipboard_set_text (win->priv->clip, timestamp, -1);
 }
@@ -88,7 +85,7 @@ clip_text_cb (GtkClipboard *clip,
 
 	if (text) {
 		timestamp = g_strdup (text);
-		pt_player_goto_timestamp (win->priv->player, timestamp);
+		pt_player_goto_timestamp (win->player, timestamp);
 		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
 		g_free (timestamp);
 	}
@@ -117,12 +114,12 @@ goto_position (GSimpleAction *action,
 	gint	      pos;
 
 	dlg = pt_goto_dialog_new (GTK_WINDOW (win));
-	pt_goto_dialog_set_pos (dlg, pt_player_get_position (win->priv->player) / 1000);
-	pt_goto_dialog_set_max (dlg, pt_player_get_duration (win->priv->player) / 1000);
+	pt_goto_dialog_set_pos (dlg, pt_player_get_position (win->player) / 1000);
+	pt_goto_dialog_set_max (dlg, pt_player_get_duration (win->player) / 1000);
 
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_OK) {
 		pos = pt_goto_dialog_get_pos (dlg);
-		pt_player_jump_to_position (win->priv->player, pos * 1000);
+		pt_player_jump_to_position (win->player, pos * 1000);
 		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
 	}
 
@@ -171,6 +168,7 @@ zoom_out (GSimpleAction *action,
 	set_zoom (win->priv->editor, -25);
 }
 
+#ifdef HAVE_ASR
 static void
 setup_sphinx (PtWindow *win)
 {
@@ -179,9 +177,9 @@ setup_sphinx (PtWindow *win)
 	pt_asr_settings_apply_config (
 			win->priv->asr_settings,
 			g_settings_get_string (win->priv->editor, "asr-config"),
-			win->priv->player);
+			win->player);
 
-	pt_player_setup_sphinx (win->priv->player, &error);
+	pt_player_setup_sphinx (win->player, &error);
 	if (error) {
 		pt_error_message (win, error->message, NULL);
 		g_clear_error (&error);
@@ -283,7 +281,7 @@ set_mode_playback (PtWindow *win)
 {
 	GError *error = NULL;
 
-	pt_player_setup_player (win->priv->player, &error);
+	pt_player_setup_player (win->player, &error);
 	if (error) {
 		pt_error_message (win, error->message, NULL);
 		g_clear_error (&error);
@@ -330,9 +328,12 @@ change_mode (GSimpleAction *action,
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->primary_menu_button)))
 		gtk_button_clicked (GTK_BUTTON (win->priv->primary_menu_button));
 }
+#endif
 
 const GActionEntry win_actions[] = {
+#ifdef HAVE_ASR
 	{ "mode", NULL, "s", "'playback'", change_mode },
+#endif
 	{ "copy", copy_timestamp, NULL, NULL, NULL },
 	{ "insert", insert_timestamp, NULL, NULL, NULL },
 	{ "goto", goto_position, NULL, NULL, NULL },
@@ -347,12 +348,12 @@ update_time (PtWindow *win)
 	gchar *text;
 	gint64 time;
 
-	time = pt_player_get_position (win->priv->player);
+	time = pt_player_get_position (win->player);
 
 	/* Update label only if time has changed at 10th seconds level */
 	if (time/100 != win->priv->last_time) {
 		text = pt_player_get_current_time_string (
-				win->priv->player,
+				win->player,
 				PT_PRECISION_SECOND_10TH);
 
 		if (text == NULL)
@@ -376,7 +377,7 @@ update_time_tick (GtkWidget     *widget,
 	return G_SOURCE_CONTINUE;
 }
 
-void
+static void
 zoom_in_cb (GtkWidget *widget,
             PtWindow  *win)
 {
@@ -386,7 +387,7 @@ zoom_in_cb (GtkWidget *widget,
 	g_action_activate (action, NULL);
 }
 
-void
+static void
 zoom_out_cb (GtkWidget *widget,
              PtWindow  *win)
 {
@@ -424,7 +425,7 @@ change_jump_back_tooltip (PtWindow *win)
 	gchar *back;
 	gint   seconds;
 
-	seconds = pt_player_get_back (win->priv->player) / 1000;
+	seconds = pt_player_get_back (win->player) / 1000;
 	back = g_strdup_printf (
 			ngettext ("Jump back 1 second",
 				  "Jump back %d seconds",
@@ -441,7 +442,7 @@ change_jump_forward_tooltip (PtWindow *win)
 	gchar *forward;
 	gint   seconds;
 
-	seconds = pt_player_get_forward (win->priv->player) / 1000;
+	seconds = pt_player_get_forward (win->player) / 1000;
 	forward = g_strdup_printf (
 			ngettext ("Jump forward 1 second",
 				  "Jump forward %d seconds",
@@ -459,7 +460,7 @@ change_play_button_tooltip (PtWindow *win)
 	gint      pause;
 	gboolean  free_me = FALSE;
 
-	pause = pt_player_get_pause (win->priv->player) / 1000;
+	pause = pt_player_get_pause (win->player) / 1000;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
 		if (pause == 0) {
 			play = _("Pause");
@@ -486,7 +487,7 @@ update_insert_action_sensitivity_cb (GtkClipboard *clip,
                                      gpointer      data)
 {
 	PtWindow *win = PT_WINDOW (data);
-	PtPlayer *player = win->priv->player;
+	PtPlayer *player = win->player;
 	gchar    *timestamp = NULL;
 	gboolean  result = FALSE;
 	GAction  *action;
@@ -563,9 +564,11 @@ enable_win_actions (PtWindow *win,
 {
 	GAction *action;
 
+#ifdef HAVE_ASR
 	/* always active */
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), TRUE);
+#endif
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "copy");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
@@ -592,59 +595,6 @@ enable_win_actions (PtWindow *win,
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 }
 
-static void
-destroy_progress_dlg (PtWindow *win)
-{
-	if (win->priv->progress_handler_id > 0) {
-		g_signal_handler_disconnect (win->priv->player, win->priv->progress_handler_id);
-		win->priv->progress_handler_id = 0;
-	}
-
-	if (win->priv->progress_dlg) {
-		gtk_widget_destroy (win->priv->progress_dlg);
-	}
-}
-
-static void
-progress_response_cb (GtkWidget *dialog,
-                      gint       response,
-                      PtWindow  *win)
-{
-	if (response == GTK_RESPONSE_CANCEL)
-		pt_player_cancel (win->priv->player);
-		/* This will trigger an error */
-}
-
-static void
-show_progress_dlg (PtWindow *win)
-{
-	if (win->priv->progress_dlg) {
-		/* actually this should never happen */
-		gtk_window_present (GTK_WINDOW (win->priv->progress_dlg));
-		return;
-	}
-
-	win->priv->progress_dlg = GTK_WIDGET (pt_progress_dialog_new (GTK_WINDOW (win)));
-
-	g_signal_connect (win->priv->progress_dlg,
-			  "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &win->priv->progress_dlg);
-
-	g_signal_connect (win->priv->progress_dlg,
-			  "response",
-			  G_CALLBACK (progress_response_cb),
-			  win);
-
-	win->priv->progress_handler_id =
-		g_signal_connect_swapped (win->priv->player,
-					  "load-progress",
-					  G_CALLBACK (pt_progress_dialog_set_progress),
-					  PT_PROGRESS_DIALOG (win->priv->progress_dlg));
-
-	gtk_widget_show_all (win->priv->progress_dlg);
-}
-
 static void
 pt_window_ready_to_play (PtWindow *win,
                          gboolean  state)
@@ -665,19 +615,19 @@ pt_window_ready_to_play (PtWindow *win,
 	gtk_widget_set_sensitive (win->priv->button_jump_back, state);
 	gtk_widget_set_sensitive (win->priv->button_jump_forward, state);
 	gtk_widget_set_sensitive (win->priv->speed_scale, state);
+	gtk_widget_set_sensitive (win->priv->volumebutton, state);
 
 	if (state) {
 		open_context = gtk_widget_get_style_context (win->priv->button_open);
 		gtk_style_context_remove_class (open_context, "suggested-action");
-		destroy_progress_dlg (win);
-		display_name = pt_player_get_filename (win->priv->player);
+		display_name = pt_player_get_filename (win->player);
 		if (display_name) {
 			gtk_window_set_title (GTK_WINDOW (win), display_name);
 			g_free (display_name);
 		}
 
 		gchar *uri = NULL;
-		uri = pt_player_get_uri (win->priv->player);
+		uri = pt_player_get_uri (win->player);
 		if (uri) {
 			gtk_recent_manager_add_item (win->priv->recent, uri);
 			g_free (uri);
@@ -687,12 +637,6 @@ pt_window_ready_to_play (PtWindow *win,
 		change_jump_back_tooltip (win);
 		change_jump_forward_tooltip (win);
 		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
-		win->priv->wavedata = pt_player_get_data (win->priv->player,
-							  g_settings_get_int (win->priv->editor, "pps"));
-		pt_waveviewer_set_wave (PT_WAVEVIEWER (win->priv->waveviewer),
-					win->priv->wavedata);
-		pt_wavedata_free (win->priv->wavedata);
-		/* add timer after waveviewer, didn’t update cursor otherwise sometimes */
 		add_timer (win);
 
 	} else {
@@ -701,7 +645,6 @@ pt_window_ready_to_play (PtWindow *win,
 		gtk_widget_set_tooltip_text (win->priv->button_jump_back, NULL);
 		gtk_widget_set_tooltip_text (win->priv->button_jump_forward, NULL);
 		remove_timer (win);
-		pt_waveviewer_set_wave (PT_WAVEVIEWER (win->priv->waveviewer), NULL);
 	}
 }
 
@@ -710,45 +653,58 @@ player_error_cb (PtPlayer *player,
                  GError   *error,
                  PtWindow *win)
 {
-	destroy_progress_dlg (win);
 	pt_window_ready_to_play (win, FALSE);
 	pt_error_message (win, error->message, NULL);
 	/* TODO clear error? */
 }
 
 static void
-open_cb (PtPlayer     *player,
+open_cb (PtWaveviewer *viewer,
 	 GAsyncResult *res,
 	 gpointer     *data)
 {
 	PtWindow *win = (PtWindow *) data;
 	GError	 *error = NULL;
 
-	destroy_progress_dlg (win);
-
-	if (!pt_player_open_uri_finish (player, res, &error)) {
+	if (!pt_waveviewer_load_wave_finish (viewer, res, &error)) {
 		pt_error_message (win, error->message, NULL);
 		g_error_free (error);
+		/* TODO that's it? */
 		return;
 	}
-
-	pt_window_ready_to_play (win, TRUE);
 }
 
 gchar*
 pt_window_get_uri (PtWindow *win)
 {
-	return pt_player_get_uri (win->priv->player);
+	return pt_player_get_uri (win->player);
 }
 
 void
 pt_window_open_file (PtWindow *win,
                      gchar    *uri)
 {
-	show_progress_dlg (win);
+	g_return_if_fail (uri != NULL);
+
+	gchar *current_uri;
+	gint cmp = 1;
+
+	/* Don't reload already loaded waveform */
+	current_uri = pt_player_get_uri (win->player);
+	if (current_uri) {
+		cmp = g_strcmp0 (current_uri, uri);
+		g_free (current_uri);
+	}
+	if (cmp == 0)
+		return;
+
 	pt_window_ready_to_play (win, FALSE);
-	pt_player_open_uri_async (win->priv->player,
+	if (!pt_player_open_uri (win->player, uri))
+		return;
+	pt_window_ready_to_play (win, TRUE);
+	pt_waveviewer_load_wave_async (PT_WAVEVIEWER (win->priv->waveviewer),
 				  uri,
+				  NULL,
 				  (GAsyncReadyCallback) open_cb,
 				  win);
 }
@@ -781,14 +737,14 @@ player_play_toggled_cb (PtPlayer *player,
 	update_play_after_toggle (win, play);
 }
 
-void
+static void
 play_button_toggled_cb (GtkToggleButton *button,
                         PtWindow        *win)
 {
 	/* GUI button toggled, block signals from PtPlayer */
-	g_signal_handlers_block_by_func (win->priv->player, player_play_toggled_cb, win);
-	pt_player_play_pause (win->priv->player);
-	g_signal_handlers_unblock_by_func (win->priv->player, player_play_toggled_cb, win);
+	g_signal_handlers_block_by_func (win->player, player_play_toggled_cb, win);
+	pt_player_play_pause (win->player);
+	g_signal_handlers_unblock_by_func (win->player, player_play_toggled_cb, win);
 
 	update_play_after_toggle (win, button);
 }
@@ -804,7 +760,7 @@ player_end_of_stream_cb (PtPlayer *player,
 	g_signal_handlers_block_by_func (play, play_button_toggled_cb, win);
 	gtk_toggle_button_set_active (play, FALSE);
 	g_signal_handlers_unblock_by_func (play, play_button_toggled_cb, win);
-	pt_player_pause (win->priv->player);
+	pt_player_pause (win->player);
 	change_play_button_tooltip (win);
 }
 
@@ -819,13 +775,13 @@ player_jumped_back_cb (PtPlayer *player,
 	g_signal_handlers_unblock_by_func (button, jump_back_button_clicked_cb, win);
 }
 
-void
+static void
 jump_back_button_clicked_cb (GtkButton *button,
                              PtWindow  *win)
 {
-	g_signal_handlers_block_by_func (win->priv->player, player_jumped_back_cb, win);
-	pt_player_jump_back (win->priv->player);
-	g_signal_handlers_unblock_by_func (win->priv->player, player_jumped_back_cb, win);
+	g_signal_handlers_block_by_func (win->player, player_jumped_back_cb, win);
+	pt_player_jump_back (win->player);
+	g_signal_handlers_unblock_by_func (win->player, player_jumped_back_cb, win);
 }
 
 static void
@@ -839,72 +795,64 @@ player_jumped_forward_cb (PtPlayer *player,
 	g_signal_handlers_unblock_by_func (button, jump_forward_button_clicked_cb, win);
 }
 
-void
+static void
 jump_forward_button_clicked_cb (GtkButton *button,
                                 PtWindow  *win)
 {
-	pt_player_jump_forward (win->priv->player);
+	pt_player_jump_forward (win->player);
 }
 
 /* currently not used */
-gboolean
+static gboolean
 fast_back_button_pressed_cb (GtkButton *button,
                              GdkEvent  *event,
                              PtWindow  *win)
 {
-	pt_player_rewind (win->priv->player, 2.0);
+	pt_player_rewind (win->player, 2.0);
 	return FALSE;
 }
 
 /* currently not used */
-gboolean
+static gboolean
 fast_back_button_released_cb (GtkButton *button,
                               GdkEvent  *event,
                               PtWindow  *win)
 {
-	pt_player_set_speed (win->priv->player, win->priv->speed);
+	pt_player_set_speed (win->player, win->priv->speed);
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
-		pt_player_play (win->priv->player);
+		pt_player_play (win->player);
 	} else {
-		pt_player_pause (win->priv->player);
+		pt_player_pause (win->player);
 	}
 	return FALSE;
 }
 
 /* currently not used */
-gboolean
+static gboolean
 fast_forward_button_pressed_cb (GtkButton *button,
                                 GdkEvent  *event,
                                 PtWindow  *win)
 {
-	pt_player_fast_forward (win->priv->player, 2.0);
+	pt_player_fast_forward (win->player, 2.0);
 	return FALSE;
 }
 
 /* currently not used */
-gboolean
+static gboolean
 fast_forward_button_released_cb (GtkButton *button,
                                  GdkEvent  *event,
                                  PtWindow  *win)
 {
-	pt_player_set_speed (win->priv->player, win->priv->speed);
+	pt_player_set_speed (win->player, win->priv->speed);
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
-		pt_player_play (win->priv->player);
+		pt_player_play (win->player);
 	} else {
-		pt_player_pause (win->priv->player);
+		pt_player_pause (win->player);
 	}
 	return FALSE;
 }
 
-void
-speed_changed_cb (GtkRange *range,
-                  PtWindow *win)
-{
-	win->priv->speed = gtk_range_get_value (range);
-	pt_player_set_speed (win->priv->player, win->priv->speed);
-}
-
-
+#ifdef HAVE_ASR
 static void
 player_asr_final_cb (PtPlayer *player,
                      gchar    *word,
@@ -920,8 +868,9 @@ player_asr_hypothesis_cb (PtPlayer *player,
 {
 	pt_asr_output_hypothesis (win->priv->output, word);
 }
+#endif
 
-void
+static void
 speed_scale_direction_changed_cb (GtkWidget        *widget,
                                   GtkTextDirection  previous_direction,
                                   gpointer          data)
@@ -952,7 +901,7 @@ swap_accelerators (GtkWidget     *widget,
 			GTK_ACCEL_VISIBLE);
 }
 
-void
+static void
 jump_back_direction_changed_cb (GtkWidget        *widget,
                                 GtkTextDirection  previous_direction,
                                 gpointer          data)
@@ -971,7 +920,7 @@ jump_back_direction_changed_cb (GtkWidget        *widget,
 	swap_accelerators (widget, self->priv->accels, old, new);
 }
 
-void
+static void
 jump_forward_direction_changed_cb (GtkWidget        *widget,
                                    GtkTextDirection  previous_direction,
                                    gpointer          data)
@@ -1009,21 +958,6 @@ settings_changed_cb (GSettings *settings,
 		change_jump_forward_tooltip (win);
 		return;
 	}
-
-	if (g_strcmp0 (key, "pps") == 0) {
-		gchar *uri;
-		uri = pt_player_get_uri (win->priv->player);
-		if (uri) {
-			win->priv->wavedata = pt_player_get_data (win->priv->player,
-								  g_settings_get_int (win->priv->editor, "pps"));
-			g_object_set (win->priv->waveviewer, "zoom", TRUE, NULL);
-			pt_waveviewer_set_wave (PT_WAVEVIEWER (win->priv->waveviewer),
-						win->priv->wavedata);
-			pt_wavedata_free (win->priv->wavedata);
-			g_free (uri);
-		}
-		return;
-	}
 }
 
 static gboolean
@@ -1050,14 +984,32 @@ map_milliseconds_to_seconds (const GValue       *value,
 	return g_variant_new_int32 (new);
 }
 
+static void
+setup_non_wayland_env (PtWindow *win)
+{
+	if (g_settings_get_boolean (win->priv->editor, "start-on-top")) {
+		gtk_window_set_keep_above (GTK_WINDOW (win), TRUE);
+	}
+	if (g_settings_get_boolean (win->priv->editor, "remember-position")) {
+		gtk_window_move (GTK_WINDOW (win),
+				 g_settings_get_int (win->priv->editor, "x-pos"),
+				 g_settings_get_int (win->priv->editor, "y-pos"));
+	}
+}
+
 static void
 setup_settings (PtWindow *win)
 {
-	win->priv->editor = g_settings_new ("com.github.gkarsay.parlatype");
+	win->priv->editor = g_settings_new (APP_ID);
+
+	g_settings_bind (
+			win->priv->editor, "pps",
+			win->priv->waveviewer, "pps",
+			G_SETTINGS_BIND_GET);
 
 	g_settings_bind_with_mapping (
 			win->priv->editor, "rewind-on-pause",
-			win->priv->player, "pause",
+			win->player, "pause",
 			G_SETTINGS_BIND_GET,
 			map_seconds_to_milliseconds,
 			map_milliseconds_to_seconds,
@@ -1065,7 +1017,7 @@ setup_settings (PtWindow *win)
 
 	g_settings_bind_with_mapping (
 			win->priv->editor, "jump-back",
-			win->priv->player, "back",
+			win->player, "back",
 			G_SETTINGS_BIND_GET,
 			map_seconds_to_milliseconds,
 			map_milliseconds_to_seconds,
@@ -1073,7 +1025,7 @@ setup_settings (PtWindow *win)
 
 	g_settings_bind_with_mapping (
 			win->priv->editor, "jump-forward",
-			win->priv->player, "forward",
+			win->player, "forward",
 			G_SETTINGS_BIND_GET,
 			map_seconds_to_milliseconds,
 			map_milliseconds_to_seconds,
@@ -1081,12 +1033,12 @@ setup_settings (PtWindow *win)
 
 	g_settings_bind (
 			win->priv->editor, "repeat-all",
-			win->priv->player, "repeat-all",
+			win->player, "repeat-all",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
 			win->priv->editor, "repeat-selection",
-			win->priv->player, "repeat-selection",
+			win->player, "repeat-selection",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
@@ -1101,22 +1053,22 @@ setup_settings (PtWindow *win)
 
 	g_settings_bind (
 			win->priv->editor, "timestamp-precision",
-			win->priv->player, "timestamp-precision",
+			win->player, "timestamp-precision",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
 			win->priv->editor, "timestamp-fixed",
-			win->priv->player, "timestamp-fixed",
+			win->player, "timestamp-fixed",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
 			win->priv->editor, "timestamp-delimiter",
-			win->priv->player, "timestamp-delimiter",
+			win->player, "timestamp-delimiter",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
 			win->priv->editor, "timestamp-fraction-sep",
-			win->priv->player, "timestamp-fraction-sep",
+			win->player, "timestamp-fraction-sep",
 			G_SETTINGS_BIND_GET);
 
 	/* connect to tooltip changer */
@@ -1133,28 +1085,103 @@ setup_settings (PtWindow *win)
 	   - Save last known speed in metadata for each file */
 	win->priv->speed = 1.0;
 
-	/* Set size on both X11 and Wayland */
 	if (g_settings_get_boolean (win->priv->editor, "remember-size")) {
 		gtk_window_resize (GTK_WINDOW (win),
 				   g_settings_get_int (win->priv->editor, "width"),
 				   g_settings_get_int (win->priv->editor, "height"));
 	}
 
-	/* The following would fail silently on Wayland, we make it conditional anyway */
 #ifdef GDK_WINDOWING_X11
 	GdkDisplay *display;
 	display = gdk_display_get_default ();
-	if (GDK_IS_X11_DISPLAY (display)) {
-		if (g_settings_get_boolean (win->priv->editor, "start-on-top")) {
-			gtk_window_set_keep_above (GTK_WINDOW (win), TRUE);
-		}
-		if (g_settings_get_boolean (win->priv->editor, "remember-position")) {
-			gtk_window_move (GTK_WINDOW (win),
-					 g_settings_get_int (win->priv->editor, "x-pos"),
-					 g_settings_get_int (win->priv->editor, "y-pos"));
-		}
-	}
+	if (GDK_IS_X11_DISPLAY (display))
+		setup_non_wayland_env (win);
 #endif
+
+#ifdef GDK_WINDOWING_WIN32
+	setup_non_wayland_env (win);
+#endif
+}
+
+static void
+volume_button_update_mute (GObject    *gobject,
+                           GParamSpec *pspec,
+                           gpointer    user_data)
+{
+	/* Switch icons of volume button depending on mute state.
+	 * If muted, only the mute icon is shown (volume can still be adjusted).
+	 * In normal state restore original icon set. */
+
+	PtWindow *win = PT_WINDOW (user_data);
+	GtkScaleButton *volumebutton = GTK_SCALE_BUTTON (win->priv->volumebutton);
+	const gchar* mute_icon [] = {(gchar*)win->priv->vol_icons[0], NULL};
+
+	if (pt_player_get_mute (win->player)) {
+		gtk_scale_button_set_icons (volumebutton, mute_icon);
+	} else {
+		gtk_scale_button_set_icons (volumebutton, (const gchar**) win->priv->vol_icons);
+	}
+}
+
+static void
+volume_button_update_volume (GObject    *gobject,
+                             GParamSpec *pspec,
+                             gpointer    user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+	GtkScaleButton *volumebutton = GTK_SCALE_BUTTON (win->priv->volumebutton);
+	gdouble volume = pt_player_get_volume (win->player);
+
+	gtk_scale_button_set_value (volumebutton, volume);
+}
+
+static gboolean
+volume_button_value_changed_cb (GtkWidget *volumebutton,
+                                gdouble    value,
+                                gpointer   user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+	pt_player_set_volume (win->player, value);
+	return FALSE;
+}
+
+static gboolean
+volume_button_event_cb (GtkWidget *volumebutton,
+                        GdkEvent  *event,
+                        gpointer   user_data)
+{
+	/* This is for pulseaudiosink in paused state. It doesn't notify of
+	 * volume/mute changes. Update values when user is interacting with
+	 * the volume button. */
+
+	PtWindow *win = PT_WINDOW (user_data);
+	gtk_scale_button_set_value (GTK_SCALE_BUTTON (volumebutton),
+	                            pt_player_get_volume (win->player));
+	volume_button_update_mute (NULL, NULL, win);
+	return FALSE;
+}
+
+static gboolean
+volume_button_press_event (GtkGestureMultiPress *gesture,
+                           gint                  n_press,
+                           gdouble               x,
+                           gdouble               y,
+                           gpointer              user_data)
+{
+	/* Switch mute state on click with secondary button */
+
+	PtWindow *win = PT_WINDOW (user_data);
+	guint     button;
+	gboolean  current_mute;
+
+	button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
+
+	if (n_press == 1 && button == GDK_BUTTON_SECONDARY) {
+		current_mute = pt_player_get_mute (win->player);
+		pt_player_set_mute (win->player, !current_mute);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 static void
@@ -1162,8 +1189,8 @@ setup_player (PtWindow *win)
 {
 	GError *error = NULL;
 
-	win->priv->player = pt_player_new ();
-	if (!pt_player_setup_player (win->priv->player, &error)) {
+	win->player = pt_player_new ();
+	if (!pt_player_setup_player (win->player, &error)) {
 		gchar *secondary_message = g_strdup_printf (
 			_("Parlatype needs GStreamer 1.x to run. Please check your installation of "
                         "GStreamer and make sure you have the “Good Plugins” installed.\n"
@@ -1175,48 +1202,103 @@ setup_player (PtWindow *win)
 	}
 
 	pt_player_connect_waveviewer (
-			win->priv->player,
+			win->player,
 			PT_WAVEVIEWER (win->priv->waveviewer));
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"end-of-stream",
 			G_CALLBACK (player_end_of_stream_cb),
 			win);
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"error",
 			G_CALLBACK (player_error_cb),
 			win);
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"play-toggled",
 			G_CALLBACK (player_play_toggled_cb),
 			win);
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"jumped-back",
 			G_CALLBACK (player_jumped_back_cb),
 			win);
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"jumped-forward",
 			G_CALLBACK (player_jumped_forward_cb),
 			win);
 
+	GtkAdjustment *speed_adjustment;
+	speed_adjustment = gtk_range_get_adjustment (GTK_RANGE (win->priv->speed_scale));
 	g_object_bind_property (
-			win->priv->volumebutton, "value",
-			win->priv->player, "volume",
+			speed_adjustment, "value",
+			win->player, "speed",
 			G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
-	g_signal_connect (win->priv->player,
+#ifdef HAVE_ASR
+	g_signal_connect (win->player,
 			"asr-final",
 			G_CALLBACK (player_asr_final_cb),
 			win);
 
-	g_signal_connect (win->priv->player,
+	g_signal_connect (win->player,
 			"asr-hypothesis",
 			G_CALLBACK (player_asr_hypothesis_cb),
 			win);
+#endif
+}
+
+static void
+setup_volume (PtWindow *win)
+{
+	/* Connect to volumebutton's "value-changed" signal and to PtPlayer's
+	 * "notify:volume" signal instead of binding the 2 properties. On
+	 * startup there is no external Pulseaudio volume yet and the binding
+	 * would impose our default volume on Pulseaudio. Instead we want to
+	 * pick up Pulseaudio's volume. That is Parlatype/application specific
+	 * and it even remembers the last value. The first external volume
+	 * signal is sent after a file was opened successfully, until then our
+	 * volume button is insensitive. */
+	g_signal_connect (win->priv->volumebutton,
+			"value-changed",
+			G_CALLBACK (volume_button_value_changed_cb),
+			win);
+
+	g_signal_connect (win->priv->volumebutton,
+			"event",
+			G_CALLBACK (volume_button_event_cb),
+			win);
+
+	/* Get complete icon set of volume button to change it depending on
+	 * mute state. */
+	g_object_get (win->priv->volumebutton,
+	              "icons", &win->priv->vol_icons, NULL);
+
+	g_signal_connect (win->player,
+			"notify::mute",
+			G_CALLBACK (volume_button_update_mute),
+			win);
+
+	g_signal_connect (win->player,
+			"notify::volume",
+			G_CALLBACK (volume_button_update_volume),
+			win);
+
+	/* Switch mute state on mouse click with secondary button */
+	win->priv->vol_event = gtk_gesture_multi_press_new (
+			win->priv->volumebutton);
+	gtk_gesture_single_set_exclusive (
+			GTK_GESTURE_SINGLE (win->priv->vol_event), TRUE);
+	gtk_gesture_single_set_button (
+			GTK_GESTURE_SINGLE (win->priv->vol_event), 0);
+	gtk_event_controller_set_propagation_phase (
+			GTK_EVENT_CONTROLLER (win->priv->vol_event), GTK_PHASE_CAPTURE);
+	g_signal_connect (win->priv->vol_event,
+			"pressed",
+			G_CALLBACK (volume_button_press_event),
+			win);
 }
 
 static void
@@ -1235,7 +1317,7 @@ setup_accels_actions_headerbar (PtWindow *win)
 	GtkWidget     *hbar;
 	GMenuModel    *primary_menu;
 
-	builder = gtk_builder_new_from_resource ("/com/github/gkarsay/parlatype/window-headerbar.ui");
+	builder = gtk_builder_new_from_resource ("/org/parlatype/parlatype/window-headerbar.ui");
 	hbar = GTK_WIDGET (gtk_builder_get_object (builder, "headerbar"));
 	win->priv->button_open = GTK_WIDGET (gtk_builder_get_object (builder, "button_open"));
 	win->priv->primary_menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "primary_menu_button"));
@@ -1243,7 +1325,7 @@ setup_accels_actions_headerbar (PtWindow *win)
 	gtk_builder_connect_signals (builder, win);
 	g_object_unref (builder);
 
-	builder = gtk_builder_new_from_resource ("/com/github/gkarsay/parlatype/menus.ui");
+	builder = gtk_builder_new_from_resource ("/org/parlatype/parlatype/menus.ui");
 	primary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "primary-menu"));
 	win->priv->secondary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "secondary-menu"));
 	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (win->priv->primary_menu_button), primary_menu);
@@ -1314,24 +1396,22 @@ pt_window_init (PtWindow *win)
 	setup_player (win);
 	setup_settings (win);
 	setup_accels_actions_headerbar (win);
-	setup_mediakeys (win);		/* this is in pt_mediakeys.c */
+	setup_volume (win);
 	pt_window_setup_dnd (win);	/* this is in pt_window_dnd.c */
-	setup_dbus_service (win);	/* this is in pt_dbus_service.c */
+#ifdef HAVE_ASR
 	win->priv->asr_settings = NULL;
 	win->priv->output = NULL;
 	win->priv->output_handler_id1 = 0;
 	win->priv->output_handler_id2 = 0;
+#endif
 	win->priv->recent = gtk_recent_manager_get_default ();
 	win->priv->timer = 0;
 	win->priv->last_time = 0;
-	win->priv->progress_dlg = NULL;
-	win->priv->progress_handler_id = 0;
-	win->priv->wavedata = NULL;
 	win->priv->clip_handler_id = 0;
 	win->priv->clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 
 	/* Used e.g. by Xfce */
-	gtk_window_set_default_icon_name ("com.github.gkarsay.parlatype");
+	gtk_window_set_default_icon_name (APP_ID);
 
 	/* Flip speed scale for right to left layouts */
 	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
@@ -1347,6 +1427,11 @@ pt_window_init (PtWindow *win)
 			"follow-cursor-changed",
 			G_CALLBACK (update_goto_cursor_action_sensitivity),
 			win);
+
+	g_signal_connect_swapped (win->priv->waveviewer,
+				  "load-progress",
+				  G_CALLBACK (gtk_progress_bar_set_fraction),
+				  GTK_PROGRESS_BAR (win->priv->progress));
 }
 
 static void
@@ -1355,8 +1440,6 @@ pt_window_dispose (GObject *object)
 	PtWindow *win;
 	win = PT_WINDOW (object);
 
-	close_dbus_service (win);
-
 	/* Save window size/position on all backends */
 	if (win->priv->editor) {
 		gint x;
@@ -1375,12 +1458,13 @@ pt_window_dispose (GObject *object)
 	}
 	remove_timer (win);
 	g_clear_object (&win->priv->editor);
-	g_clear_object (&win->priv->proxy);
-	destroy_progress_dlg (win);
-	g_clear_object (&win->priv->player);
+	g_clear_object (&win->player);
+#ifdef HAVE_ASR
 	g_clear_object (&win->priv->output);
 	g_clear_object (&win->priv->asr_settings);
+#endif
 	g_clear_object (&win->priv->go_to_timestamp);
+	g_clear_object (&win->priv->vol_event);
 
 	G_OBJECT_CLASS (pt_window_parent_class)->dispose (object);
 }
@@ -1392,7 +1476,20 @@ pt_window_class_init (PtWindowClass *klass)
 	GtkWidgetClass *widget_class  = GTK_WIDGET_CLASS (klass);
 
 	gobject_class->dispose      = pt_window_dispose;
-	gtk_widget_class_set_template_from_resource (widget_class, "/com/github/gkarsay/parlatype/window.ui");
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/parlatype/parlatype/window.ui");
+	gtk_widget_class_bind_template_callback(widget_class, fast_back_button_pressed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, fast_back_button_released_cb);
+	gtk_widget_class_bind_template_callback(widget_class, fast_forward_button_pressed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, fast_forward_button_released_cb);
+	gtk_widget_class_bind_template_callback(widget_class, jump_back_button_clicked_cb);
+	gtk_widget_class_bind_template_callback(widget_class, jump_back_direction_changed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, jump_forward_button_clicked_cb);
+	gtk_widget_class_bind_template_callback(widget_class, jump_forward_direction_changed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, play_button_toggled_cb);
+	gtk_widget_class_bind_template_callback(widget_class, speed_scale_direction_changed_cb);
+	gtk_widget_class_bind_template_callback(widget_class, zoom_in_cb);
+	gtk_widget_class_bind_template_callback(widget_class, zoom_out_cb);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, progress);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_play);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_back);	// not used
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_forward);	// not used
